"""
本地 MCP 服务管理器
解决 FastAPI 环境下无法启动本地命令服务的问题
"""

import asyncio
import logging
import os
import signal
import subprocess
import sys
import time
from pathlib import Path
from typing import Dict, List, Optional, Tuple, Any
from dataclasses import dataclass
try:
    import psutil
except ImportError:
    psutil = None
import tempfile
import json

logger = logging.getLogger(__name__)

@dataclass
class LocalServiceProcess:
    """本地服务进程信息"""
    name: str
    process: subprocess.Popen
    config: Dict[str, Any]
    start_time: float
    pid: int
    status: str = "running"
    restart_count: int = 0
    last_health_check: float = 0

class LocalServiceManager:
    """本地 MCP 服务管理器"""
    
    def __init__(self, base_work_dir: str = None):
        self.processes: Dict[str, LocalServiceProcess] = {}
        self.base_work_dir = Path(base_work_dir or os.getcwd())
        self.temp_dir = Path(tempfile.gettempdir()) / "mcpstore_local_services"
        self.temp_dir.mkdir(exist_ok=True)
        
        # 健康检查配置
        self.health_check_interval = 30  # 30秒
        self.max_restart_attempts = 3
        self.restart_delay = 5  # 5秒
        
        # 启动健康检查任务
        self._health_check_task = None
        # 延迟启动健康监控，避免在没有事件循环时创建任务
        self._monitor_started = False
    
    def _start_health_monitor(self):
        """启动健康监控任务"""
        try:
            # 检查是否有运行的事件循环
            loop = asyncio.get_running_loop()
            if self._health_check_task is None or self._health_check_task.done():
                self._health_check_task = asyncio.create_task(self._health_monitor_loop())
                self._monitor_started = True
        except RuntimeError:
            # 没有运行的事件循环，延迟启动
            self._monitor_started = False
    
    def _ensure_health_monitor_started(self):
        """确保健康监控已启动"""
        if not self._monitor_started:
            self._start_health_monitor()

    async def _health_monitor_loop(self):
        """健康监控循环"""
        while True:
            try:
                await asyncio.sleep(self.health_check_interval)
                await self._check_all_services_health()
            except Exception as e:
                logger.error(f"Health monitor error: {e}")
    
    async def _check_all_services_health(self):
        """检查所有服务健康状态"""
        for service_name, service_proc in list(self.processes.items()):
            try:
                if not self._is_process_alive(service_proc):
                    logger.warning(f"Service {service_name} is not running, attempting restart...")
                    await self._restart_service(service_name)
            except Exception as e:
                logger.error(f"Health check failed for {service_name}: {e}")
    
    def _is_process_alive(self, service_proc: LocalServiceProcess) -> bool:
        """检查进程是否存活"""
        try:
            # 检查进程是否存在
            if service_proc.process.poll() is not None:
                return False
            
            # 使用 psutil 进行更详细的检查（如果可用）
            if psutil:
                try:
                    proc = psutil.Process(service_proc.pid)
                    return proc.is_running() and proc.status() != psutil.STATUS_ZOMBIE
                except psutil.NoSuchProcess:
                    return False
            else:
                # 如果没有 psutil，使用基本检查
                return True
                
        except Exception as e:
            logger.debug(f"Process check error: {e}")
            return False
    
    async def start_local_service(self, name: str, config: Dict[str, Any]) -> Tuple[bool, str]:
        """
        启动本地 MCP 服务

        Args:
            name: 服务名称
            config: 服务配置，包含 command, args, env, working_dir 等

        Returns:
            Tuple[bool, str]: (是否成功, 消息)
        """
        try:
            # 确保健康监控已启动
            self._ensure_health_monitor_started()
            # 检查是否已经运行
            if name in self.processes and self._is_process_alive(self.processes[name]):
                return True, f"Service {name} is already running"
            
            # 验证配置
            if "command" not in config:
                return False, "Missing required 'command' field"
            
            # 准备启动参数
            command = config["command"]
            args = config.get("args", [])
            env = self._prepare_environment(config.get("env", {}))
            working_dir = self._resolve_working_dir(config.get("working_dir"))
            
            # 构建完整命令
            full_command = [command] + args
            
            logger.info(f"Starting local service {name}: {' '.join(full_command)}")
            logger.debug(f"Working directory: {working_dir}")
            logger.debug(f"Environment: {env}")
            
            # 启动进程
            process = subprocess.Popen(
                full_command,
                cwd=working_dir,
                env=env,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
                text=True,
                bufsize=1,
                universal_newlines=True
            )
            
            # 等待一小段时间确保进程启动
            await asyncio.sleep(1)
            
            # 检查进程是否成功启动
            if process.poll() is not None:
                stdout, stderr = process.communicate()
                error_msg = f"Process failed to start. Exit code: {process.returncode}"
                if stderr:
                    error_msg += f"\nStderr: {stderr}"
                return False, error_msg
            
            # 记录进程信息
            service_proc = LocalServiceProcess(
                name=name,
                process=process,
                config=config,
                start_time=time.time(),
                pid=process.pid
            )
            
            self.processes[name] = service_proc
            
            logger.info(f"Local service {name} started successfully (PID: {process.pid})")
            return True, f"Service started successfully (PID: {process.pid})"
            
        except Exception as e:
            logger.error(f"Failed to start local service {name}: {e}")
            return False, str(e)
    
    def _prepare_environment(self, custom_env: Dict[str, str]) -> Dict[str, str]:
        """准备环境变量"""
        # 🔧 修改：支持独立环境配置
        try:
            # 尝试获取独立配置管理器
            from mcpstore.core.standalone_config import get_global_config
            config_manager = get_global_config()

            if config_manager and config_manager.config.isolated_environment:
                # 使用独立环境配置
                env = config_manager.get_isolated_environment(custom_env)
                logger.info("Using isolated environment from standalone config")
            else:
                # 使用传统环境继承
                env = os.environ.copy()
                env.update(custom_env)
                logger.debug("Using inherited environment variables")
        except Exception as e:
            # 回退到传统方式
            logger.warning(f"Failed to use standalone config, falling back to inherited env: {e}")
            env = os.environ.copy()
            env.update(custom_env)

        # 确保 Python 路径正确
        if "PYTHONPATH" not in env:
            env["PYTHONPATH"] = str(self.base_work_dir)
        else:
            env["PYTHONPATH"] = f"{self.base_work_dir}:{env['PYTHONPATH']}"

        return env
    
    def _resolve_working_dir(self, working_dir: Optional[str]) -> str:
        """解析工作目录"""
        if working_dir:
            # 如果是相对路径，相对于 base_work_dir
            work_path = Path(working_dir)
            if not work_path.is_absolute():
                work_path = self.base_work_dir / work_path
            return str(work_path.resolve())
        else:
            return str(self.base_work_dir)
    
    async def stop_local_service(self, name: str) -> Tuple[bool, str]:
        """停止本地服务"""
        try:
            if name not in self.processes:
                return False, f"Service {name} not found"
            
            service_proc = self.processes[name]
            
            # 尝试优雅关闭
            try:
                service_proc.process.terminate()
                
                # 等待进程结束
                try:
                    service_proc.process.wait(timeout=10)
                except subprocess.TimeoutExpired:
                    # 强制杀死
                    service_proc.process.kill()
                    service_proc.process.wait()
                
                logger.info(f"Local service {name} stopped successfully")
                
            except Exception as e:
                logger.warning(f"Error stopping service {name}: {e}")
            
            # 清理记录
            del self.processes[name]
            return True, "Service stopped successfully"
            
        except Exception as e:
            logger.error(f"Failed to stop local service {name}: {e}")
            return False, str(e)
    
    async def _restart_service(self, name: str) -> bool:
        """重启服务"""
        try:
            if name not in self.processes:
                return False
            
            service_proc = self.processes[name]
            
            # 检查重启次数
            if service_proc.restart_count >= self.max_restart_attempts:
                logger.error(f"Service {name} exceeded max restart attempts")
                service_proc.status = "failed"
                return False
            
            # 停止服务
            await self.stop_local_service(name)
            
            # 等待一段时间
            await asyncio.sleep(self.restart_delay)
            
            # 重新启动
            success, message = await self.start_local_service(name, service_proc.config)
            
            if success:
                self.processes[name].restart_count = service_proc.restart_count + 1
                logger.info(f"Service {name} restarted successfully (attempt {service_proc.restart_count + 1})")
                return True
            else:
                logger.error(f"Failed to restart service {name}: {message}")
                return False
                
        except Exception as e:
            logger.error(f"Error restarting service {name}: {e}")
            return False
    
    def get_service_status(self, name: str) -> Dict[str, Any]:
        """获取服务状态"""
        if name not in self.processes:
            return {"status": "not_found"}
        
        service_proc = self.processes[name]
        is_alive = self._is_process_alive(service_proc)
        
        return {
            "status": "running" if is_alive else "stopped",
            "pid": service_proc.pid,
            "start_time": service_proc.start_time,
            "restart_count": service_proc.restart_count,
            "uptime": time.time() - service_proc.start_time if is_alive else 0
        }
    
    def list_services(self) -> Dict[str, Dict[str, Any]]:
        """列出所有服务状态"""
        return {name: self.get_service_status(name) for name in self.processes}
    
    async def cleanup(self):
        """清理所有服务"""
        logger.info("Cleaning up local services...")
        
        # 停止健康监控
        if self._health_check_task:
            self._health_check_task.cancel()
        
        # 停止所有服务
        for name in list(self.processes.keys()):
            await self.stop_local_service(name)
        
        logger.info("Local service cleanup completed")

# 全局实例
_local_service_manager: Optional[LocalServiceManager] = None

def get_local_service_manager() -> LocalServiceManager:
    """获取全局本地服务管理器实例"""
    global _local_service_manager
    if _local_service_manager is None:
        _local_service_manager = LocalServiceManager()
    return _local_service_manager



#!/usr/bin/env python3
"""
组件控制与过滤 (Component Control)
基于标签的动态过滤，支持启用/禁用组件，创建环境配置文件
"""

import logging
from typing import Dict, List, Set, Any, Optional, Union
from dataclasses import dataclass, field
from enum import Enum
import json
from pathlib import Path

logger = logging.getLogger(__name__)

class ComponentType(Enum):
    """组件类型"""
    TOOL = "tool"
    RESOURCE = "resource"
    PROMPT = "prompt"
    SERVICE = "service"

class EnvironmentType(Enum):
    """环境类型"""
    DEVELOPMENT = "development"
    TESTING = "testing"
    STAGING = "staging"
    PRODUCTION = "production"
    CUSTOM = "custom"

@dataclass
class ComponentInfo:
    """组件信息"""
    name: str
    component_type: ComponentType
    tags: Set[str] = field(default_factory=set)
    enabled: bool = True
    service_name: Optional[str] = None
    description: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)

@dataclass
class EnvironmentProfile:
    """环境配置文件"""
    name: str
    environment_type: EnvironmentType
    allowed_tags: Set[str] = field(default_factory=set)
    blocked_tags: Set[str] = field(default_factory=set)
    allowed_components: Set[str] = field(default_factory=set)
    blocked_components: Set[str] = field(default_factory=set)
    component_overrides: Dict[str, bool] = field(default_factory=dict)  # 组件启用/禁用覆盖
    description: Optional[str] = None

class ComponentFilter:
    """组件过滤器"""
    
    def __init__(self):
        self._components: Dict[str, ComponentInfo] = {}
        self._tag_index: Dict[str, Set[str]] = {}  # tag -> component_names
        self._type_index: Dict[ComponentType, Set[str]] = {}  # type -> component_names
    
    def register_component(self, component: ComponentInfo):
        """注册组件"""
        self._components[component.name] = component
        
        # 更新标签索引
        for tag in component.tags:
            if tag not in self._tag_index:
                self._tag_index[tag] = set()
            self._tag_index[tag].add(component.name)
        
        # 更新类型索引
        if component.component_type not in self._type_index:
            self._type_index[component.component_type] = set()
        self._type_index[component.component_type].add(component.name)
        
        logger.debug(f"Registered component: {component.name} ({component.component_type.value})")
    
    def filter_by_tags(self, include_tags: List[str] = None, exclude_tags: List[str] = None) -> List[ComponentInfo]:
        """基于标签过滤组件"""
        include_tags = set(include_tags or [])
        exclude_tags = set(exclude_tags or [])
        
        result = []
        for component in self._components.values():
            # 检查包含标签
            if include_tags and not include_tags.intersection(component.tags):
                continue
            
            # 检查排除标签
            if exclude_tags and exclude_tags.intersection(component.tags):
                continue
            
            # 检查是否启用
            if not component.enabled:
                continue
            
            result.append(component)
        
        return result
    
    def filter_by_type(self, component_type: ComponentType) -> List[ComponentInfo]:
        """按类型过滤组件"""
        component_names = self._type_index.get(component_type, set())
        return [self._components[name] for name in component_names if self._components[name].enabled]
    
    def get_components_by_service(self, service_name: str) -> List[ComponentInfo]:
        """获取指定服务的组件"""
        return [
            component for component in self._components.values()
            if component.service_name == service_name and component.enabled
        ]
    
    def enable_component(self, component_name: str, enabled: bool = True):
        """启用/禁用组件"""
        if component_name in self._components:
            self._components[component_name].enabled = enabled
            logger.info(f"Component {component_name} {'enabled' if enabled else 'disabled'}")
        else:
            logger.warning(f"Component {component_name} not found")
    
    def bulk_enable_components(self, component_names: List[str], enabled: bool = True):
        """批量启用/禁用组件"""
        for name in component_names:
            self.enable_component(name, enabled)
    
    def get_component_info(self, component_name: str) -> Optional[ComponentInfo]:
        """获取组件信息"""
        return self._components.get(component_name)
    
    def list_all_tags(self) -> List[str]:
        """列出所有标签"""
        return list(self._tag_index.keys())
    
    def get_components_with_tag(self, tag: str) -> List[ComponentInfo]:
        """获取具有指定标签的组件"""
        component_names = self._tag_index.get(tag, set())
        return [self._components[name] for name in component_names]

class EnvironmentManager:
    """环境管理器"""
    
    def __init__(self, config_dir: Optional[Path] = None):
        self.config_dir = config_dir or Path.home() / ".mcpstore" / "environments"
        self.config_dir.mkdir(parents=True, exist_ok=True)
        self._profiles: Dict[str, EnvironmentProfile] = {}
        self._current_profile: Optional[str] = None
        self._load_default_profiles()
    
    def _load_default_profiles(self):
        """加载默认环境配置"""
        # 开发环境：允许所有工具
        dev_profile = EnvironmentProfile(
            name="development",
            environment_type=EnvironmentType.DEVELOPMENT,
            allowed_tags={"development", "testing", "debug", "experimental"},
            description="Development environment with all tools enabled"
        )
        self._profiles["development"] = dev_profile
        
        # 生产环境：只允许安全的工具
        prod_profile = EnvironmentProfile(
            name="production",
            environment_type=EnvironmentType.PRODUCTION,
            allowed_tags={"production", "safe", "stable"},
            blocked_tags={"experimental", "debug", "dangerous"},
            description="Production environment with only safe, stable tools"
        )
        self._profiles["production"] = prod_profile
        
        # 测试环境
        test_profile = EnvironmentProfile(
            name="testing",
            environment_type=EnvironmentType.TESTING,
            allowed_tags={"testing", "safe", "mock"},
            blocked_tags={"production-only", "dangerous"},
            description="Testing environment with mock and safe tools"
        )
        self._profiles["testing"] = test_profile
    
    def create_profile(self, profile: EnvironmentProfile):
        """创建环境配置文件"""
        self._profiles[profile.name] = profile
        self._save_profile(profile)
        logger.info(f"Created environment profile: {profile.name}")
    
    def load_profile(self, profile_name: str) -> Optional[EnvironmentProfile]:
        """加载环境配置文件"""
        if profile_name in self._profiles:
            return self._profiles[profile_name]
        
        # 尝试从文件加载
        profile_file = self.config_dir / f"{profile_name}.json"
        if profile_file.exists():
            try:
                with open(profile_file, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    profile = self._dict_to_profile(data)
                    self._profiles[profile_name] = profile
                    return profile
            except Exception as e:
                logger.error(f"Failed to load profile {profile_name}: {e}")
        
        return None
    
    def activate_profile(self, profile_name: str) -> bool:
        """激活环境配置文件"""
        profile = self.load_profile(profile_name)
        if profile:
            self._current_profile = profile_name
            logger.info(f"Activated environment profile: {profile_name}")
            return True
        else:
            logger.error(f"Profile {profile_name} not found")
            return False
    
    def get_current_profile(self) -> Optional[EnvironmentProfile]:
        """获取当前环境配置"""
        if self._current_profile:
            return self._profiles.get(self._current_profile)
        return None
    
    def apply_profile_to_filter(self, component_filter: ComponentFilter, profile_name: Optional[str] = None):
        """将环境配置应用到组件过滤器"""
        profile = self._profiles.get(profile_name or self._current_profile)
        if not profile:
            logger.warning("No profile to apply")
            return
        
        # 应用组件启用/禁用覆盖
        for component_name, enabled in profile.component_overrides.items():
            component_filter.enable_component(component_name, enabled)
        
        # 根据标签禁用组件
        if profile.blocked_tags:
            for tag in profile.blocked_tags:
                components = component_filter.get_components_with_tag(tag)
                for component in components:
                    component_filter.enable_component(component.name, False)
        
        logger.info(f"Applied profile {profile.name} to component filter")
    
    def list_profiles(self) -> List[str]:
        """列出所有环境配置文件"""
        return list(self._profiles.keys())
    
    def _save_profile(self, profile: EnvironmentProfile):
        """保存环境配置文件"""
        profile_file = self.config_dir / f"{profile.name}.json"
        try:
            with open(profile_file, 'w', encoding='utf-8') as f:
                json.dump(self._profile_to_dict(profile), f, indent=2, ensure_ascii=False)
        except Exception as e:
            logger.error(f"Failed to save profile {profile.name}: {e}")
    
    def _profile_to_dict(self, profile: EnvironmentProfile) -> Dict[str, Any]:
        """将配置文件转换为字典"""
        return {
            "name": profile.name,
            "environment_type": profile.environment_type.value,
            "allowed_tags": list(profile.allowed_tags),
            "blocked_tags": list(profile.blocked_tags),
            "allowed_components": list(profile.allowed_components),
            "blocked_components": list(profile.blocked_components),
            "component_overrides": profile.component_overrides,
            "description": profile.description
        }
    
    def _dict_to_profile(self, data: Dict[str, Any]) -> EnvironmentProfile:
        """将字典转换为配置文件"""
        return EnvironmentProfile(
            name=data["name"],
            environment_type=EnvironmentType(data["environment_type"]),
            allowed_tags=set(data.get("allowed_tags", [])),
            blocked_tags=set(data.get("blocked_tags", [])),
            allowed_components=set(data.get("allowed_components", [])),
            blocked_components=set(data.get("blocked_components", [])),
            component_overrides=data.get("component_overrides", {}),
            description=data.get("description")
        )

class ComponentControlManager:
    """组件控制管理器"""
    
    def __init__(self):
        self.filter = ComponentFilter()
        self.environment_manager = EnvironmentManager()
    
    def register_tool(self, name: str, service_name: str, tags: List[str] = None, **metadata):
        """注册工具组件"""
        component = ComponentInfo(
            name=name,
            component_type=ComponentType.TOOL,
            tags=set(tags or []),
            service_name=service_name,
            metadata=metadata
        )
        self.filter.register_component(component)
    
    def get_available_tools(self, environment: Optional[str] = None, tags: List[str] = None) -> List[ComponentInfo]:
        """获取可用工具（考虑环境和标签过滤）"""
        if environment:
            self.environment_manager.activate_profile(environment)
            self.environment_manager.apply_profile_to_filter(self.filter, environment)
        
        if tags:
            return self.filter.filter_by_tags(include_tags=tags)
        else:
            return self.filter.filter_by_type(ComponentType.TOOL)
    
    def create_custom_environment(self, name: str, allowed_tags: List[str], blocked_tags: List[str] = None):
        """创建自定义环境"""
        profile = EnvironmentProfile(
            name=name,
            environment_type=EnvironmentType.CUSTOM,
            allowed_tags=set(allowed_tags),
            blocked_tags=set(blocked_tags or []),
            description=f"Custom environment: {name}"
        )
        self.environment_manager.create_profile(profile)
    
    def switch_environment(self, environment_name: str) -> bool:
        """切换环境"""
        return self.environment_manager.activate_profile(environment_name)

# 全局实例
_global_component_manager = None

def get_component_manager() -> ComponentControlManager:
    """获取全局组件控制管理器"""
    global _global_component_manager
    if _global_component_manager is None:
        _global_component_manager = ComponentControlManager()
    return _global_component_manager



#!/usr/bin/env python3
"""
OpenAPI 深度集成
自动化 API 转换，自定义路由映射，智能生成 MCP 组件名称
"""

import logging
import re
import httpx
from typing import Dict, List, Any, Optional, Union, Tuple
from dataclasses import dataclass, field
from enum import Enum
from urllib.parse import urlparse
import json

logger = logging.getLogger(__name__)

class MCPComponentType(Enum):
    """MCP 组件类型"""
    TOOL = "tool"
    RESOURCE = "resource"
    RESOURCE_TEMPLATE = "resource_template"

class HTTPMethod(Enum):
    """HTTP 方法"""
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"
    PATCH = "PATCH"
    HEAD = "HEAD"
    OPTIONS = "OPTIONS"

@dataclass
class RouteMapping:
    """路由映射配置"""
    path_pattern: str                    # 路径模式，支持正则表达式
    method: Optional[HTTPMethod] = None  # HTTP 方法，None 表示匹配所有方法
    mcp_type: MCPComponentType = MCPComponentType.TOOL  # 映射到的 MCP 组件类型
    name_template: Optional[str] = None  # 名称模板
    description_template: Optional[str] = None  # 描述模板
    tags: List[str] = field(default_factory=list)  # 标签

@dataclass
class OpenAPIServiceConfig:
    """OpenAPI 服务配置"""
    name: str
    spec_url: str
    base_url: Optional[str] = None
    auth_config: Optional[Dict[str, Any]] = None
    route_mappings: List[RouteMapping] = field(default_factory=list)
    custom_names: Dict[str, str] = field(default_factory=dict)  # operation_id -> custom_name
    global_tags: List[str] = field(default_factory=list)
    auto_sync: bool = False  # 是否自动同步 API 变更

class OpenAPIAnalyzer:
    """OpenAPI 规范分析器"""
    
    def __init__(self):
        self._spec_cache: Dict[str, Dict[str, Any]] = {}
    
    async def fetch_spec(self, spec_url: str) -> Dict[str, Any]:
        """获取 OpenAPI 规范"""
        if spec_url in self._spec_cache:
            return self._spec_cache[spec_url]
        
        try:
            async with httpx.AsyncClient() as client:
                response = await client.get(spec_url)
                response.raise_for_status()
                spec = response.json()
                self._spec_cache[spec_url] = spec
                logger.info(f"Fetched OpenAPI spec from {spec_url}")
                return spec
        except Exception as e:
            logger.error(f"Failed to fetch OpenAPI spec from {spec_url}: {e}")
            raise
    
    def analyze_endpoints(self, spec: Dict[str, Any]) -> List[Dict[str, Any]]:
        """分析 API 端点"""
        endpoints = []
        paths = spec.get("paths", {})
        
        for path, path_item in paths.items():
            for method, operation in path_item.items():
                if method.upper() not in [m.value for m in HTTPMethod]:
                    continue
                
                endpoint_info = {
                    "path": path,
                    "method": method.upper(),
                    "operation_id": operation.get("operationId"),
                    "summary": operation.get("summary"),
                    "description": operation.get("description"),
                    "tags": operation.get("tags", []),
                    "parameters": operation.get("parameters", []),
                    "request_body": operation.get("requestBody"),
                    "responses": operation.get("responses", {}),
                    "security": operation.get("security", [])
                }
                endpoints.append(endpoint_info)
        
        return endpoints
    
    def suggest_mcp_type(self, endpoint: Dict[str, Any]) -> MCPComponentType:
        """建议 MCP 组件类型"""
        method = endpoint["method"]
        path = endpoint["path"]
        
        # GET 请求通常映射为 Resource
        if method == "GET":
            # 如果路径包含参数，映射为 ResourceTemplate
            if "{" in path and "}" in path:
                return MCPComponentType.RESOURCE_TEMPLATE
            else:
                return MCPComponentType.RESOURCE
        
        # 其他方法映射为 Tool
        return MCPComponentType.TOOL
    
    def generate_component_name(self, endpoint: Dict[str, Any], custom_names: Dict[str, str] = None) -> str:
        """生成组件名称"""
        operation_id = endpoint.get("operation_id")
        
        # 使用自定义名称
        if custom_names and operation_id and operation_id in custom_names:
            return custom_names[operation_id]
        
        # 使用 operation_id（截断到第一个双下划线）
        if operation_id:
            name = operation_id.split("__")[0]
            return self._slugify_name(name)
        
        # 根据路径和方法生成名称
        method = endpoint["method"].lower()
        path = endpoint["path"]
        
        # 清理路径
        path_parts = [part for part in path.split("/") if part and not part.startswith("{")]
        if path_parts:
            resource = "_".join(path_parts)
        else:
            resource = "api"
        
        name = f"{method}_{resource}"
        return self._slugify_name(name)
    
    def _slugify_name(self, name: str) -> str:
        """将名称转换为合法的标识符"""
        # 转换为小写
        name = name.lower()
        # 替换特殊字符为下划线
        name = re.sub(r'[^a-z0-9_]', '_', name)
        # 移除连续的下划线
        name = re.sub(r'_+', '_', name)
        # 移除开头和结尾的下划线
        name = name.strip('_')
        # 限制长度
        if len(name) > 56:
            name = name[:56].rstrip('_')
        
        return name or "unnamed"

class RouteMapper:
    """路由映射器"""
    
    def __init__(self):
        self._default_mappings = self._create_default_mappings()
    
    def _create_default_mappings(self) -> List[RouteMapping]:
        """创建默认路由映射"""
        return [
            # GET 请求映射为 Resource
            RouteMapping(
                path_pattern=r".*",
                method=HTTPMethod.GET,
                mcp_type=MCPComponentType.RESOURCE,
                tags=["read-only"]
            ),
            # POST/PUT/DELETE 映射为 Tool
            RouteMapping(
                path_pattern=r".*",
                method=HTTPMethod.POST,
                mcp_type=MCPComponentType.TOOL,
                tags=["write"]
            ),
            RouteMapping(
                path_pattern=r".*",
                method=HTTPMethod.PUT,
                mcp_type=MCPComponentType.TOOL,
                tags=["write", "update"]
            ),
            RouteMapping(
                path_pattern=r".*",
                method=HTTPMethod.DELETE,
                mcp_type=MCPComponentType.TOOL,
                tags=["write", "delete", "destructive"]
            )
        ]
    
    def apply_mappings(self, endpoint: Dict[str, Any], custom_mappings: List[RouteMapping] = None) -> Tuple[MCPComponentType, List[str]]:
        """应用路由映射"""
        mappings = custom_mappings or self._default_mappings
        path = endpoint["path"]
        method = HTTPMethod(endpoint["method"])
        
        for mapping in mappings:
            # 检查路径模式
            if not re.match(mapping.path_pattern, path):
                continue
            
            # 检查方法
            if mapping.method and mapping.method != method:
                continue
            
            # 匹配成功
            return mapping.mcp_type, mapping.tags
        
        # 默认映射
        if method == HTTPMethod.GET:
            return MCPComponentType.RESOURCE, ["read-only"]
        else:
            return MCPComponentType.TOOL, ["write"]

class OpenAPIIntegrationManager:
    """OpenAPI 集成管理器"""
    
    def __init__(self):
        self.analyzer = OpenAPIAnalyzer()
        self.route_mapper = RouteMapper()
        self._services: Dict[str, OpenAPIServiceConfig] = {}
    
    def register_openapi_service(self, config: OpenAPIServiceConfig):
        """注册 OpenAPI 服务"""
        self._services[config.name] = config
        logger.info(f"Registered OpenAPI service: {config.name}")
    
    async def import_openapi_service(
        self,
        name: str,
        spec_url: str,
        base_url: Optional[str] = None,
        route_mappings: List[RouteMapping] = None,
        custom_names: Dict[str, str] = None
    ) -> Dict[str, Any]:
        """导入 OpenAPI 服务"""
        
        # 获取规范
        spec = await self.analyzer.fetch_spec(spec_url)
        
        # 分析端点
        endpoints = self.analyzer.analyze_endpoints(spec)
        
        # 生成 MCP 组件
        components = []
        for endpoint in endpoints:
            # 应用路由映射
            mcp_type, tags = self.route_mapper.apply_mappings(endpoint, route_mappings)
            
            # 生成组件名称
            component_name = self.analyzer.generate_component_name(endpoint, custom_names)
            
            component = {
                "name": component_name,
                "type": mcp_type.value,
                "endpoint": endpoint,
                "tags": tags + (endpoint.get("tags", [])),
                "description": endpoint.get("description") or endpoint.get("summary"),
                "service_name": name
            }
            components.append(component)
        
        # 创建服务配置
        service_config = OpenAPIServiceConfig(
            name=name,
            spec_url=spec_url,
            base_url=base_url or self._extract_base_url(spec),
            route_mappings=route_mappings or [],
            custom_names=custom_names or {}
        )
        self.register_openapi_service(service_config)
        
        result = {
            "service_name": name,
            "spec_info": {
                "title": spec.get("info", {}).get("title"),
                "version": spec.get("info", {}).get("version"),
                "description": spec.get("info", {}).get("description")
            },
            "components": components,
            "total_endpoints": len(endpoints),
            "component_types": {
                "tools": len([c for c in components if c["type"] == "tool"]),
                "resources": len([c for c in components if c["type"] == "resource"]),
                "resource_templates": len([c for c in components if c["type"] == "resource_template"])
            }
        }
        
        logger.info(f"Imported OpenAPI service {name}: {len(components)} components generated")
        return result
    
    async def sync_service_changes(self, service_name: str) -> Dict[str, Any]:
        """同步服务变更"""
        if service_name not in self._services:
            raise ValueError(f"Service {service_name} not found")
        
        config = self._services[service_name]
        
        # 重新获取规范
        new_spec = await self.analyzer.fetch_spec(config.spec_url)
        new_endpoints = self.analyzer.analyze_endpoints(new_spec)
        
        # 比较变更
        # 这里可以实现更复杂的变更检测逻辑
        
        return {
            "service_name": service_name,
            "changes_detected": True,  # 简化实现
            "new_endpoints_count": len(new_endpoints)
        }
    
    def create_custom_route_mapping(
        self,
        service_name: str,
        path_patterns: Dict[str, MCPComponentType]
    ) -> List[RouteMapping]:
        """创建自定义路由映射"""
        mappings = []
        for pattern, mcp_type in path_patterns.items():
            mapping = RouteMapping(
                path_pattern=pattern,
                mcp_type=mcp_type,
                tags=["custom-mapped"]
            )
            mappings.append(mapping)
        
        return mappings
    
    def get_service_info(self, service_name: str) -> Optional[OpenAPIServiceConfig]:
        """获取服务信息"""
        return self._services.get(service_name)
    
    def list_services(self) -> List[str]:
        """列出所有服务"""
        return list(self._services.keys())
    
    def _extract_base_url(self, spec: Dict[str, Any]) -> Optional[str]:
        """从规范中提取基础 URL"""
        servers = spec.get("servers", [])
        if servers:
            return servers[0].get("url")
        return None

# 全局实例
_global_openapi_manager = None

def get_openapi_manager() -> OpenAPIIntegrationManager:
    """获取全局 OpenAPI 集成管理器"""
    global _global_openapi_manager
    if _global_openapi_manager is None:
        _global_openapi_manager = OpenAPIIntegrationManager()
    return _global_openapi_manager



"""
智能重连管理器
实现指数退避重连策略，支持重连优先级和失败计数
"""

import asyncio
import logging
from datetime import datetime, timedelta
from typing import Dict, Set, Optional, Tuple
from dataclasses import dataclass
from enum import Enum

logger = logging.getLogger(__name__)


class ReconnectionPriority(Enum):
    """重连优先级"""
    LOW = 1      # 低优先级：非关键服务
    NORMAL = 2   # 普通优先级：一般服务
    HIGH = 3     # 高优先级：关键服务
    CRITICAL = 4 # 关键优先级：核心服务


@dataclass
class ReconnectionEntry:
    """重连条目"""
    service_key: str                    # 服务键 (client_id:service_name)
    client_id: str                      # 客户端ID
    service_name: str                   # 服务名称
    priority: ReconnectionPriority      # 重连优先级
    failure_count: int = 0              # 失败次数
    last_attempt: Optional[datetime] = None  # 最后尝试时间
    next_attempt: Optional[datetime] = None  # 下次尝试时间
    created_at: datetime = None         # 创建时间
    
    def __post_init__(self):
        if self.created_at is None:
            self.created_at = datetime.now()


class SmartReconnectionManager:
    """智能重连管理器"""
    
    def __init__(self):
        self.entries: Dict[str, ReconnectionEntry] = {}
        
        # 重连策略配置
        self.base_delay_seconds = 60        # 基础延迟：1分钟
        self.max_delay_seconds = 600        # 最大延迟：10分钟
        self.max_failure_count = 10         # 最大失败次数
        self.cleanup_interval_hours = 24    # 清理间隔：24小时
        
        # 优先级权重（影响重连间隔）
        self.priority_weights = {
            ReconnectionPriority.CRITICAL: 0.5,  # 关键服务：更快重连
            ReconnectionPriority.HIGH: 0.7,      # 高优先级：较快重连
            ReconnectionPriority.NORMAL: 1.0,    # 普通优先级：标准重连
            ReconnectionPriority.LOW: 1.5        # 低优先级：较慢重连
        }
    
    def add_service(self, client_id: str, service_name: str, 
                   priority: ReconnectionPriority = ReconnectionPriority.NORMAL) -> str:
        """添加服务到重连队列"""
        service_key = f"{client_id}:{service_name}"
        
        if service_key in self.entries:
            # 如果已存在，增加失败计数
            entry = self.entries[service_key]
            entry.failure_count += 1
            self._calculate_next_attempt(entry)
            logger.debug(f"Updated reconnection entry for {service_key}, failure_count: {entry.failure_count}")
        else:
            # 创建新条目
            entry = ReconnectionEntry(
                service_key=service_key,
                client_id=client_id,
                service_name=service_name,
                priority=priority
            )
            self._calculate_next_attempt(entry)
            self.entries[service_key] = entry
            logger.info(f"Added new reconnection entry for {service_key} with priority {priority.name}")
        
        return service_key
    
    def remove_service(self, service_key: str) -> bool:
        """从重连队列中移除服务"""
        if service_key in self.entries:
            del self.entries[service_key]
            logger.info(f"Removed reconnection entry for {service_key}")
            return True
        return False
    
    def mark_success(self, service_key: str) -> bool:
        """标记服务重连成功"""
        return self.remove_service(service_key)
    
    def mark_failure(self, service_key: str) -> bool:
        """标记服务重连失败"""
        if service_key in self.entries:
            entry = self.entries[service_key]
            entry.failure_count += 1
            entry.last_attempt = datetime.now()
            
            # 检查是否超过最大失败次数
            if entry.failure_count >= self.max_failure_count:
                logger.warning(f"Service {service_key} exceeded max failure count ({self.max_failure_count}), removing from queue")
                self.remove_service(service_key)
                return False
            
            # 重新计算下次尝试时间
            self._calculate_next_attempt(entry)
            logger.debug(f"Marked failure for {service_key}, failure_count: {entry.failure_count}, next_attempt: {entry.next_attempt}")
            return True
        return False
    
    def get_services_ready_for_retry(self) -> list[ReconnectionEntry]:
        """获取准备重试的服务列表（按优先级排序）"""
        now = datetime.now()
        ready_services = []
        
        for entry in self.entries.values():
            if entry.next_attempt and entry.next_attempt <= now:
                ready_services.append(entry)
        
        # 按优先级排序（优先级高的先重连）
        ready_services.sort(key=lambda x: (x.priority.value, x.failure_count), reverse=True)
        
        return ready_services
    
    def get_queue_status(self) -> Dict:
        """获取重连队列状态"""
        now = datetime.now()
        status = {
            "total_entries": len(self.entries),
            "ready_for_retry": 0,
            "by_priority": {priority.name: 0 for priority in ReconnectionPriority},
            "by_failure_count": {},
            "oldest_entry": None,
            "next_retry_time": None
        }
        
        next_retry_times = []
        
        for entry in self.entries.values():
            # 统计优先级分布
            status["by_priority"][entry.priority.name] += 1
            
            # 统计失败次数分布
            failure_key = f"{entry.failure_count}_failures"
            status["by_failure_count"][failure_key] = status["by_failure_count"].get(failure_key, 0) + 1
            
            # 检查是否准备重试
            if entry.next_attempt and entry.next_attempt <= now:
                status["ready_for_retry"] += 1
            
            # 收集下次重试时间
            if entry.next_attempt:
                next_retry_times.append(entry.next_attempt)
            
            # 找到最旧的条目
            if status["oldest_entry"] is None or entry.created_at < status["oldest_entry"]:
                status["oldest_entry"] = entry.created_at
        
        # 找到最近的重试时间
        if next_retry_times:
            status["next_retry_time"] = min(next_retry_times)
        
        return status
    
    def cleanup_expired_entries(self) -> int:
        """清理过期的重连条目"""
        cutoff_time = datetime.now() - timedelta(hours=self.cleanup_interval_hours)
        expired_keys = []
        
        for service_key, entry in self.entries.items():
            if entry.created_at < cutoff_time:
                expired_keys.append(service_key)
        
        for key in expired_keys:
            del self.entries[key]
        
        if expired_keys:
            logger.info(f"Cleaned up {len(expired_keys)} expired reconnection entries")
        
        return len(expired_keys)
    
    def cleanup_invalid_clients(self, valid_client_ids: Set[str]) -> int:
        """清理无效客户端的重连条目"""
        invalid_keys = []
        
        for service_key, entry in self.entries.items():
            if entry.client_id not in valid_client_ids:
                invalid_keys.append(service_key)
        
        for key in invalid_keys:
            del self.entries[key]
        
        if invalid_keys:
            logger.info(f"Cleaned up {len(invalid_keys)} reconnection entries for invalid clients")
        
        return len(invalid_keys)
    
    def _calculate_next_attempt(self, entry: ReconnectionEntry):
        """计算下次尝试时间（指数退避）"""
        # 基础延迟 * 2^失败次数 * 优先级权重
        delay_seconds = min(
            self.base_delay_seconds * (2 ** entry.failure_count) * self.priority_weights[entry.priority],
            self.max_delay_seconds
        )
        
        entry.next_attempt = datetime.now() + timedelta(seconds=delay_seconds)
        entry.last_attempt = datetime.now()
        
        logger.debug(f"Calculated next attempt for {entry.service_key}: {entry.next_attempt} "
                    f"(delay: {delay_seconds}s, failures: {entry.failure_count}, priority: {entry.priority.name})")
    
    def _infer_service_priority(self, service_name: str) -> ReconnectionPriority:
        """根据服务名称推断优先级"""
        service_name_lower = service_name.lower()
        
        # 关键服务
        if any(keyword in service_name_lower for keyword in ['auth', 'security', 'core', 'main']):
            return ReconnectionPriority.CRITICAL
        
        # 高优先级服务
        if any(keyword in service_name_lower for keyword in ['api', 'gateway', 'proxy']):
            return ReconnectionPriority.HIGH
        
        # 低优先级服务
        if any(keyword in service_name_lower for keyword in ['test', 'debug', 'temp', 'sample']):
            return ReconnectionPriority.LOW
        
        # 默认普通优先级
        return ReconnectionPriority.NORMAL



#!/usr/bin/env python3
"""
MCPStore Configuration Manager - 配置文件管理工具
"""
import json
import os
import typer
from pathlib import Path
from typing import Dict, Any, Optional

def get_default_config_path() -> Path:
    """获取默认配置文件路径"""
    # 优先级：当前目录 > 用户目录 > 系统目录
    paths = [
        Path.cwd() / "mcp.json",
        Path.home() / ".mcpstore" / "mcp.json",
        Path("/etc/mcpstore/mcp.json") if os.name != 'nt' else Path(os.environ.get('PROGRAMDATA', 'C:\\ProgramData')) / "mcpstore" / "mcp.json"
    ]
    
    for path in paths:
        if path.exists():
            return path
    
    # 如果都不存在，返回当前目录
    return paths[0]

def get_default_config() -> Dict[str, Any]:
    """获取默认配置"""
    return {
        "mcpServers": {
            "example-service": {
                "command": "python",
                "args": ["-m", "example_mcp_server"],
                "env": {},
                "description": "Example MCP service"
            }
        },
        "version": "0.2.0",
        "description": "MCPStore default configuration"
    }

def load_config(path: Optional[str] = None) -> Dict[str, Any]:
    """加载配置文件"""
    if path:
        config_path = Path(path)
    else:
        config_path = get_default_config_path()
    
    if not config_path.exists():
        typer.echo(f"⚠️  Configuration file not found: {config_path}")
        return {}
    
    try:
        with open(config_path, 'r', encoding='utf-8') as f:
            config = json.load(f)
        typer.echo(f"✅ Configuration loaded from: {config_path}")
        return config
    except json.JSONDecodeError as e:
        typer.echo(f"❌ Invalid JSON in config file: {e}")
        return {}
    except Exception as e:
        typer.echo(f"❌ Failed to load config: {e}")
        return {}

def save_config(config: Dict[str, Any], path: Optional[str] = None) -> bool:
    """保存配置文件"""
    if path:
        config_path = Path(path)
    else:
        config_path = get_default_config_path()
    
    try:
        # 确保目录存在
        config_path.parent.mkdir(parents=True, exist_ok=True)
        
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(config, f, indent=2, ensure_ascii=False)
        
        typer.echo(f"✅ Configuration saved to: {config_path}")
        return True
    except Exception as e:
        typer.echo(f"❌ Failed to save config: {e}")
        return False

def validate_config(config: Dict[str, Any]) -> bool:
    """验证配置文件格式"""
    errors = []
    
    # 检查必需字段
    if "mcpServers" not in config:
        errors.append("Missing 'mcpServers' field")
    else:
        servers = config["mcpServers"]
        if not isinstance(servers, dict):
            errors.append("'mcpServers' must be an object")
        else:
            for name, server_config in servers.items():
                if not isinstance(server_config, dict):
                    errors.append(f"Server '{name}' config must be an object")
                    continue
                
                # 检查服务配置
                if "command" not in server_config:
                    errors.append(f"Server '{name}' missing 'command' field")
                
                if "args" in server_config and not isinstance(server_config["args"], list):
                    errors.append(f"Server '{name}' 'args' must be a list")
                
                if "env" in server_config and not isinstance(server_config["env"], dict):
                    errors.append(f"Server '{name}' 'env' must be an object")
    
    if errors:
        typer.echo("❌ Configuration validation failed:")
        for error in errors:
            typer.echo(f"   • {error}")
        return False
    else:
        typer.echo("✅ Configuration is valid")
        return True

def show_config(path: Optional[str] = None):
    """显示配置文件内容"""
    config = load_config(path)
    
    if not config:
        typer.echo("No configuration found")
        return
    
    typer.echo("\n📋 Current Configuration:")
    typer.echo("─" * 50)
    
    # 显示基本信息
    version = config.get("version", "unknown")
    description = config.get("description", "No description")
    typer.echo(f"Version: {version}")
    typer.echo(f"Description: {description}")
    
    # 显示服务列表
    servers = config.get("mcpServers", {})
    typer.echo(f"\n🔧 MCP Services ({len(servers)} configured):")
    
    if not servers:
        typer.echo("   No services configured")
    else:
        for name, server_config in servers.items():
            command = server_config.get("command", "unknown")
            args = server_config.get("args", [])
            desc = server_config.get("description", "No description")
            
            typer.echo(f"\n   📦 {name}")
            typer.echo(f"      Command: {command}")
            if args:
                typer.echo(f"      Args: {' '.join(args)}")
            typer.echo(f"      Description: {desc}")
            
            # 显示环境变量
            env = server_config.get("env", {})
            if env:
                typer.echo(f"      Environment:")
                for key, value in env.items():
                    typer.echo(f"        {key}={value}")

def init_config(path: Optional[str] = None, force: bool = False):
    """初始化默认配置文件"""
    if path:
        config_path = Path(path)
    else:
        config_path = get_default_config_path()
    
    if config_path.exists() and not force:
        typer.echo(f"⚠️  Configuration file already exists: {config_path}")
        typer.echo("Use --force to overwrite")
        return
    
    default_config = get_default_config()
    
    if save_config(default_config, str(config_path)):
        typer.echo("🎉 Default configuration initialized!")
        typer.echo(f"📁 Location: {config_path}")
        typer.echo("\n💡 You can now edit the configuration file to add your MCP services.")

def handle_config(action: str, path: Optional[str] = None):
    """处理配置命令"""
    if action == "show":
        show_config(path)
    elif action == "validate":
        config = load_config(path)
        if config:
            validate_config(config)
        else:
            typer.echo("❌ No configuration to validate")
    elif action == "init":
        force = typer.confirm("Overwrite existing configuration?") if path and Path(path).exists() else False
        init_config(path, force)
    else:
        typer.echo(f"❌ Unknown action: {action}")
        typer.echo("Available actions: show, validate, init")

if __name__ == "__main__":
    # 简单的命令行接口用于测试
    import sys
    
    if len(sys.argv) < 2:
        typer.echo("Usage: python config_manager.py <action> [path]")
        typer.echo("Actions: show, validate, init")
        sys.exit(1)
    
    action = sys.argv[1]
    path = sys.argv[2] if len(sys.argv) > 2 else None
    
    handle_config(action, path)



from pydantic import BaseModel, Field
from typing import Optional, List, Dict, Any
from .common import RegistrationResponse

class ClientRegistrationRequest(BaseModel):
    client_id: Optional[str] = Field(None, description="客户端ID")
    service_names: Optional[List[str]] = Field(None, description="服务名列表")

# ClientRegistrationResponse 已移动到 common.py 中，请直接从 common.py 导入



import os
import sys

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import logging
from typing import Dict, Any

logger = logging.getLogger(__name__)

class LoggingConfig:
    """日志配置管理器"""

    _debug_enabled = False
    _configured = False

    @classmethod
    def setup_logging(cls, debug: bool = False, force_reconfigure: bool = False):
        """
        设置日志配置

        Args:
            debug: 是否启用调试日志
            force_reconfigure: 是否强制重新配置
        """
        if cls._configured and not force_reconfigure:
            # 如果已经配置过且不强制重新配置，只更新日志级别
            if debug != cls._debug_enabled:
                cls._set_log_level(debug)
            return

        # 配置日志格式
        if debug:
            log_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            log_level = logging.DEBUG
        else:
            log_format = '%(levelname)s - %(message)s'
            log_level = logging.ERROR  # 非调试模式只显示错误

        # 获取根日志器
        root_logger = logging.getLogger()

        # 清除现有的处理器
        for handler in root_logger.handlers[:]:
            root_logger.removeHandler(handler)

        # 创建新的处理器
        handler = logging.StreamHandler()
        formatter = logging.Formatter(log_format)
        handler.setFormatter(formatter)

        # 设置日志级别
        root_logger.setLevel(log_level)
        handler.setLevel(log_level)

        # 添加处理器
        root_logger.addHandler(handler)

        # 设置特定模块的日志级别
        cls._configure_module_loggers(debug)

        cls._debug_enabled = debug
        cls._configured = True

    @classmethod
    def _set_log_level(cls, debug: bool):
        """设置日志级别"""
        if debug:
            log_level = logging.DEBUG
        else:
            log_level = logging.ERROR  # 非调试模式只显示错误

        # 更新根日志器级别
        root_logger = logging.getLogger()
        root_logger.setLevel(log_level)

        # 更新所有处理器级别
        for handler in root_logger.handlers:
            handler.setLevel(log_level)

        # 更新特定模块的日志级别
        cls._configure_module_loggers(debug)

        cls._debug_enabled = debug

    @classmethod
    def _configure_module_loggers(cls, debug: bool):
        """配置特定模块的日志器"""
        if debug:
            # 调试模式：显示所有 MCPStore 相关日志
            mcpstore_loggers = [
                'mcpstore',
                'mcpstore.core',
                'mcpstore.core.store',
                'mcpstore.core.context',
                'mcpstore.core.orchestrator',
                'mcpstore.core.registry',
                'mcpstore.core.client_manager',
                'mcpstore.core.session_manager',
                'mcpstore.core.tool_resolver',
                'mcpstore.plugins.json_mcp',
                'mcpstore.adapters.langchain_adapter'
            ]

            for logger_name in mcpstore_loggers:
                module_logger = logging.getLogger(logger_name)
                module_logger.setLevel(logging.DEBUG)
        else:
            # 非调试模式：只显示警告和错误
            mcpstore_loggers = [
                'mcpstore',
                'mcpstore.core',
                'mcpstore.core.store',
                'mcpstore.core.context',
                'mcpstore.core.orchestrator',
                'mcpstore.core.registry',
                'mcpstore.core.client_manager',
                'mcpstore.core.session_manager',
                'mcpstore.core.tool_resolver',
                'mcpstore.plugins.json_mcp',
                'mcpstore.adapters.langchain_adapter'
            ]

            for logger_name in mcpstore_loggers:
                module_logger = logging.getLogger(logger_name)
                module_logger.setLevel(logging.ERROR)  # 非调试模式只显示错误

    @classmethod
    def is_debug_enabled(cls) -> bool:
        """检查是否启用了调试模式"""
        return cls._debug_enabled

    @classmethod
    def enable_debug(cls):
        """启用调试模式"""
        cls.setup_logging(debug=True, force_reconfigure=True)

    @classmethod
    def disable_debug(cls):
        """禁用调试模式"""
        cls.setup_logging(debug=False, force_reconfigure=True)

# --- Configuration Constants (default values) ---
# 核心监控配置
HEARTBEAT_INTERVAL_SECONDS = 60  # 心跳检查间隔（秒）
HTTP_TIMEOUT_SECONDS = 10        # HTTP请求超时（秒）
RECONNECTION_INTERVAL_SECONDS = 60  # 重连尝试间隔（秒）

# HTTP端点配置
STREAMABLE_HTTP_ENDPOINT = "/mcp"  # 流式HTTP端点路径

# @dataclass
# class LLMConfig:
#     provider: str = "openai_compatible"
#     api_key: str = ""
#     model: str = ""
#     base_url: Optional[str] = None

# def load_llm_config() -> LLMConfig:
#     """从环境变量加载LLM配置（仅支持openai兼容接口）"""
#     api_key = os.environ.get("OPENAI_API_KEY", "")
#     model = os.environ.get("OPENAI_MODEL", "")
#     base_url = os.environ.get("OPENAI_BASE_URL")
#     provider = "openai_compatible"
#     if not api_key:
#         logger.warning("OPENAI_API_KEY not set in environment.")
#     if not model:
#         logger.warning("OPENAI_MODEL not set in environment.")
#     return LLMConfig(provider=provider, api_key=api_key, model=model, base_url=base_url)

def _get_env_int(var: str, default: int) -> int:
    try:
        return int(os.environ.get(var, default))
    except Exception:
        logger.warning(f"环境变量{var}格式错误，使用默认值{default}")
        return default

def _get_env_bool(var: str, default: bool) -> bool:
    val = os.environ.get(var)
    if val is None:
        return default
    return val.lower() in ("1", "true", "yes", "on")

def load_app_config() -> Dict[str, Any]:
    """从环境变量加载全局配置"""
    config_data = {
        # 核心监控配置
        "heartbeat_interval": _get_env_int("HEARTBEAT_INTERVAL_SECONDS", HEARTBEAT_INTERVAL_SECONDS),
        "http_timeout": _get_env_int("HTTP_TIMEOUT_SECONDS", HTTP_TIMEOUT_SECONDS),
        "reconnection_interval": _get_env_int("RECONNECTION_INTERVAL_SECONDS", RECONNECTION_INTERVAL_SECONDS),

        # HTTP端点配置
        "streamable_http_endpoint": os.environ.get("STREAMABLE_HTTP_ENDPOINT", STREAMABLE_HTTP_ENDPOINT),
    }
    # 加载LLM配置
    # config_data["llm_config"] = load_llm_config()
    # logger.info(f"Loaded configuration from environment: {config_data}")
    return config_data



"""
MCPStore API 应用工厂
支持使用指定的MCPStore实例创建API应用
"""

import logging
import time
from contextlib import asynccontextmanager
from fastapi import Request, FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from fastapi.exceptions import RequestValidationError
from fastapi.responses import JSONResponse

from mcpstore.core.store import MCPStore

# 全局store实例（由MCPStore.start_api_server设置）
_global_store_instance: MCPStore = None

logger = logging.getLogger(__name__)

def get_store() -> MCPStore:
    """获取当前的MCPStore实例"""
    global _global_store_instance
    
    if _global_store_instance is None:
        # 如果没有设置全局实例，使用默认配置创建
        logger.warning("No global store instance found, creating default store")
        _global_store_instance = MCPStore.setup_store()
    
    return _global_store_instance

@asynccontextmanager
async def lifespan(app: FastAPI):
    """应用生命周期管理"""
    store = get_store()
    
    logger.info("Initializing MCPStore API service...")
    
    if store.is_using_data_space():
        workspace_dir = store.get_workspace_dir()
        logger.info(f"Using data space: {workspace_dir}")
    else:
        logger.info("Using default configuration")
    
    # 设置编排器
    try:
        await store.orchestrator.setup()
        logger.info("MCPStore API service initialized successfully")
    except Exception as e:
        logger.error(f"Failed to setup orchestrator: {e}")
        raise
    
    yield  # 应用运行期间
    
    # 应用关闭时的清理
    logger.info("Shutting down MCPStore API service...")
    
    try:
        # 清理编排器资源
        await store.orchestrator.cleanup()
        logger.info("MCPStore API service shutdown completed")
    except Exception as e:
        logger.error(f"Error during shutdown: {e}")

def create_app() -> FastAPI:
    """
    创建FastAPI应用实例
    
    Returns:
        FastAPI: 配置好的应用实例
    """
    store = get_store()
    
    # 创建应用实例
    app = FastAPI(
        title="MCPStore API",
        description="MCPStore HTTP API Service",
        version="0.2.0",
        lifespan=lifespan
    )
    
    # 配置CORS
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )
    
    # 导入并注册路由
    from .api import router
    app.include_router(router)
    
    # 注册异常处理
    @app.exception_handler(RequestValidationError)
    async def validation_exception_handler(request: Request, exc: RequestValidationError):
        errors = exc.errors()
        error_messages = []
        for error in errors:
            loc = " -> ".join([str(l) for l in error["loc"] if l != "body"])
            msg = error["msg"]
            error_messages.append(f"{loc}: {msg}")

        return JSONResponse(
            status_code=400,
            content={
                "success": False,
                "message": "Validation error",
                "data": error_messages
            }
        )

    @app.exception_handler(Exception)
    async def general_exception_handler(request: Request, exc: Exception):
        logger.error(f"Unhandled exception in {request.method} {request.url.path}: {exc}")
        return JSONResponse(
            status_code=500,
            content={
                "success": False,
                "message": "Internal server error",
                "data": str(exc)
            }
        )

    # 添加请求日志和性能监控中间件
    @app.middleware("http")
    async def log_requests_and_monitor(request: Request, call_next):
        """记录请求日志并监控性能"""
        start_time = time.time()
        
        # 增加活跃连接数
        try:
            store.for_store().increment_active_connections()
        except:
            pass  # 忽略监控错误
        
        try:
            response = await call_next(request)
            process_time = (time.time() - start_time) * 1000
            
            # 记录API调用
            try:
                store.for_store().record_api_call(process_time)
            except:
                pass  # 忽略监控错误

            # 只记录错误和较慢的请求
            if response.status_code >= 400 or process_time > 1000:
                logger.info(
                    f"{request.method} {request.url.path} - "
                    f"Status: {response.status_code}, Duration: {process_time:.2f}ms"
                )
            return response
        except Exception as e:
            process_time = (time.time() - start_time) * 1000
            logger.error(
                f"{request.method} {request.url.path} - "
                f"Error: {e}, Duration: {process_time:.2f}ms"
            )
            raise
        finally:
            # 减少活跃连接数
            try:
                store.for_store().decrement_active_connections()
            except:
                pass  # 忽略监控错误
    
    # 添加健康检查端点
    @app.get("/health")
    async def health_check():
        """健康检查端点"""
        try:
            store = get_store()
            workspace_info = None
            
            if store.is_using_data_space():
                workspace_info = {
                    "workspace_dir": store.get_workspace_dir(),
                    "mcp_config_path": store.config.json_path
                }
            
            return {
                "status": "healthy",
                "service": "MCPStore API",
                "version": "0.2.0",
                "timestamp": time.time(),
                "data_space": workspace_info
            }
        except Exception as e:
            logger.error(f"Health check failed: {e}")
            return JSONResponse(
                status_code=503,
                content={
                    "status": "unhealthy",
                    "service": "MCPStore API",
                    "error": str(e),
                    "timestamp": time.time()
                }
            )
    
    # 添加数据空间信息端点
    @app.get("/workspace/info")
    async def workspace_info():
        """获取工作空间信息"""
        try:
            store = get_store()
            
            if store.is_using_data_space():
                space_info = store.get_data_space_info()
                return {
                    "success": True,
                    "data": space_info,
                    "message": "Workspace information retrieved successfully"
                }
            else:
                return {
                    "success": True,
                    "data": {
                        "using_data_space": False,
                        "mcp_config_path": store.config.json_path,
                        "message": "Using default configuration"
                    },
                    "message": "Default workspace information"
                }
        except Exception as e:
            logger.error(f"Failed to get workspace info: {e}")
            return JSONResponse(
                status_code=500,
                content={
                    "success": False,
                    "message": f"Failed to get workspace info: {str(e)}",
                    "data": {}
                }
            )
    
    return app

# 为了向后兼容，保留原有的app实例
app = create_app()



"""
MCPStore 数据模型统一导入模块

提供所有数据模型的统一导入接口，避免重复定义和导入混乱。
"""

# 服务相关模型
from .service import (
    ServiceInfo,
    ServiceInfoResponse,
    ServicesResponse,
    RegisterRequestUnion,
    JsonUpdateRequest,
    ServiceConfig,
    URLServiceConfig,
    CommandServiceConfig,
    MCPServerConfig,
    ServiceConfigUnion,
    AddServiceRequest,
    TransportType
)

# 工具相关模型
from .tool import (
    ToolInfo,
    ToolsResponse,
    ToolExecutionRequest
)

# 客户端相关模型
from .client import (
    ClientRegistrationRequest
)

# 通用响应模型
from .common import (
    BaseResponse,
    APIResponse,
    ListResponse,
    DataResponse,
    RegistrationResponse,
    ExecutionResponse,
    ConfigResponse,
    HealthResponse
)

# 配置管理相关
try:
    from ..unified_config import UnifiedConfigManager, ConfigType, ConfigInfo
except ImportError:
    # 避免循环导入问题
    pass

# 导出所有模型，方便外部导入
__all__ = [
    # 服务模型
    'ServiceInfo',
    'ServiceInfoResponse',
    'ServicesResponse',
    'RegisterRequestUnion',
    'JsonUpdateRequest',
    'ServiceConfig',
    'URLServiceConfig',
    'CommandServiceConfig',
    'MCPServerConfig',
    'ServiceConfigUnion',
    'AddServiceRequest',
    'TransportType',

    # 工具模型
    'ToolInfo',
    'ToolsResponse',
    'ToolExecutionRequest',

    # 客户端模型
    'ClientRegistrationRequest',

    # 通用响应模型
    'BaseResponse',
    'APIResponse',
    'ListResponse',
    'DataResponse',
    'RegistrationResponse',
    'ExecutionResponse',
    'ConfigResponse',
    'HealthResponse'
]



# src/mcpstore/adapters/langchain_adapter.py

import json
from typing import Type, List, TYPE_CHECKING
from langchain_core.tools import Tool
from pydantic import BaseModel, create_model
from ..core.async_sync_helper import get_global_helper

# 使用 TYPE_CHECKING 和字符串提示来避免循环导入
if TYPE_CHECKING:
    from ..core.context import MCPStoreContext
    from ..core.models.tool import ToolInfo

class LangChainAdapter:
    """
    MCPStore 与 LangChain 之间的适配器（桥梁）。
    它将 mcpstore 的原生对象转换为 LangChain 可以直接使用的对象。
    """
    def __init__(self, context: 'MCPStoreContext'):
        self._context = context
        self._sync_helper = get_global_helper()

    def _enhance_description(self, tool_info: 'ToolInfo') -> str:
        """
        (前端防御) 增强工具描述，在 Prompt 中明确指导 LLM 使用正确的参数。
        """
        base_description = tool_info.description
        schema_properties = tool_info.inputSchema.get("properties", {})
        
        if not schema_properties:
            return base_description

        param_descriptions = []
        for param_name, param_info in schema_properties.items():
            param_type = param_info.get("type", "string")
            param_desc = param_info.get("description", "")
            param_descriptions.append(
                f"- {param_name} ({param_type}): {param_desc}"
            )
        
        # 将参数说明追加到主描述后
        enhanced_desc = base_description + "\n\n参数说明:\n" + "\n".join(param_descriptions)
        return enhanced_desc

    def _create_args_schema(self, tool_info: 'ToolInfo') -> Type[BaseModel]:
        """(数据转换) 根据 ToolInfo 的 inputSchema 动态创建 Pydantic 模型。"""
        schema_properties = tool_info.inputSchema.get("properties", {})
        type_mapping = {
            "string": str, "number": float, "integer": int, 
            "boolean": bool, "array": list, "object": dict
        }
        
        fields = {
            name: (type_mapping.get(prop.get("type", "string"), str), ...)
            for name, prop in schema_properties.items()
        }
        
        return create_model(
            f'{tool_info.name.capitalize().replace("_", "")}Input',
            **fields
        )

    def _create_tool_function(self, tool_name: str, args_schema: Type[BaseModel]):
        """
        (后端守卫) 创建一个健壮的同步执行函数，以应对 LangChain 不同的调用方式。
        """
        def _tool_executor(*args, **kwargs):
            tool_input = {}
            try:
                # 优先处理关键字参数 (e.g., func(query='北京'))
                if kwargs:
                    tool_input = kwargs
                # 其次处理位置参数
                elif args:
                    # 如果第一个位置参数是字典，直接使用 (e.g., func({'query':'北京'}))
                    if isinstance(args[0], dict):
                        tool_input = args[0]
                    # 如果是单个值，智能地映射到 schema 的第一个字段 (e.g., func('北京'))
                    else:
                        schema_fields = args_schema.model_json_schema()['properties']
                        first_field_name = next(iter(schema_fields))
                        tool_input = {first_field_name: args[0]}

                # 使用 Pydantic 模型严格验证参数，如果名称或类型不匹配会在此处报错
                validated_args = args_schema(**tool_input)
                # 调用 mcpstore 的核心方法（使用同步版本）
                result = self._context.use_tool(tool_name, validated_args.model_dump())

                # 提取实际结果
                if hasattr(result, 'result') and result.result is not None:
                    actual_result = result.result
                elif hasattr(result, 'success') and result.success:
                    actual_result = getattr(result, 'data', str(result))
                else:
                    actual_result = str(result)

                if isinstance(actual_result, (dict, list)):
                    return json.dumps(actual_result, ensure_ascii=False)
                return str(actual_result)
            except Exception as e:
                return f"执行工具 '{tool_name}' 时出错: {e}。收到的参数为: args={args}, kwargs={kwargs}"
        return _tool_executor

    async def _create_tool_coroutine(self, tool_name: str, args_schema: Type[BaseModel]):
        """
        (后端守卫) 创建一个健壮的异步执行函数，以应对 LangChain 不同的调用方式。
        """
        async def _tool_executor(*args, **kwargs):
            tool_input = {}
            try:
                # 优先处理关键字参数 (e.g., func(query='北京'))
                if kwargs:
                    tool_input = kwargs
                # 其次处理位置参数
                elif args:
                    # 如果第一个位置参数是字典，直接使用 (e.g., func({'query':'北京'}))
                    if isinstance(args[0], dict):
                        tool_input = args[0]
                    # 如果是单个值，智能地映射到 schema 的第一个字段 (e.g., func('北京'))
                    else:
                        schema_fields = args_schema.model_json_schema()['properties']
                        first_field_name = next(iter(schema_fields))
                        tool_input = {first_field_name: args[0]}
                
                # 使用 Pydantic 模型严格验证参数，如果名称或类型不匹配会在此处报错
                validated_args = args_schema(**tool_input)
                # 调用 mcpstore 的核心方法（使用异步版本，因为这个函数本身就是异步的）
                result = await self._context.use_tool_async(tool_name, validated_args.model_dump())

                # 提取实际结果
                if hasattr(result, 'result') and result.result is not None:
                    actual_result = result.result
                elif hasattr(result, 'success') and result.success:
                    actual_result = getattr(result, 'data', str(result))
                else:
                    actual_result = str(result)

                if isinstance(actual_result, (dict, list)):
                    return json.dumps(actual_result, ensure_ascii=False)
                return str(actual_result)
            except Exception as e:
                return f"执行工具 '{tool_name}' 时出错: {e}。收到的参数为: args={args}, kwargs={kwargs}"
        return _tool_executor

    def list_tools(self) -> List[Tool]:
        """获取所有可用的 mcpstore 工具，并将其转换为 LangChain Tool 列表（同步版本）。"""
        return self._sync_helper.run_async(self.list_tools_async())

    async def list_tools_async(self) -> List[Tool]:
        """获取所有可用的 mcpstore 工具，并将其转换为 LangChain Tool 列表（异步版本）。"""
        mcp_tools_info = await self._context.list_tools_async()
        langchain_tools = []
        for tool_info in mcp_tools_info:
            enhanced_description = self._enhance_description(tool_info)
            args_schema = self._create_args_schema(tool_info)

            # 创建同步和异步函数
            sync_func = self._create_tool_function(tool_info.name, args_schema)
            async_coroutine = await self._create_tool_coroutine(tool_info.name, args_schema)

            langchain_tools.append(
                Tool(
                    name=tool_info.name,
                    description=enhanced_description,
                    func=sync_func,  # 提供同步函数
                    coroutine=async_coroutine,  # 提供异步函数
                    args_schema=args_schema,
                )
            )
        return langchain_tools



"""
MCPStore Context Module
提供 MCPStore 的上下文管理功能
"""

from typing import Dict, List, Optional, Any, Union, TYPE_CHECKING
from enum import Enum
from pathlib import Path
from mcpstore.core.models.tool import ToolExecutionRequest, ToolInfo
from mcpstore.core.models.common import ExecutionResponse
from mcpstore.core.models.service import (
    ServiceInfo, AddServiceRequest, ServiceConfigUnion,
    URLServiceConfig, CommandServiceConfig, MCPServerConfig
)
import logging
from .exceptions import ServiceNotFoundError, InvalidConfigError, DeleteServiceError
from .async_sync_helper import get_global_helper

# 导入新功能模块
from .tool_transformation import get_transformation_manager
from .component_control import get_component_manager
from .openapi_integration import get_openapi_manager
from .auth_security import get_auth_manager
from .cache_performance import get_performance_optimizer
from .monitoring_analytics import get_monitoring_manager
from .monitoring import MonitoringManager, PerformanceMetrics, ToolUsageStats, AlertInfo, NetworkEndpoint, SystemResourceInfo

# 创建logger实例
logger = logging.getLogger(__name__)

if TYPE_CHECKING:
    from ..adapters.langchain_adapter import LangChainAdapter
    from .unified_config import UnifiedConfigManager

class ContextType(Enum):
    """上下文类型"""
    STORE = "store"
    AGENT = "agent"

class MCPStoreContext:
    """
    MCPStore上下文类
    负责处理具体的业务操作，维护操作的上下文环境
    """
    def __init__(self, store: 'MCPStore', agent_id: Optional[str] = None):
        self._store = store
        self._agent_id = agent_id
        self._context_type = ContextType.STORE if agent_id is None else ContextType.AGENT

        # 异步/同步兼容助手
        self._sync_helper = get_global_helper()

        # 新功能管理器
        self._transformation_manager = get_transformation_manager()
        self._component_manager = get_component_manager()
        self._openapi_manager = get_openapi_manager()
        self._auth_manager = get_auth_manager()
        self._performance_optimizer = get_performance_optimizer()
        self._monitoring_manager = get_monitoring_manager()

        # 监控管理器 - 使用数据空间管理器或默认路径
        if hasattr(self._store, '_data_space_manager') and self._store._data_space_manager:
            # 使用数据空间管理器的路径
            data_dir = self._store._data_space_manager.get_file_path("monitoring").parent
        else:
            # 使用默认路径（向后兼容）
            config_dir = Path(self._store.config.json_path).parent
            data_dir = config_dir / "monitoring"

        self._monitoring = MonitoringManager(data_dir)

        # 扩展预留
        self._metadata: Dict[str, Any] = {}
        self._config: Dict[str, Any] = {}
        self._cache: Dict[str, Any] = {}

    def for_langchain(self) -> 'LangChainAdapter':
        """返回一个 LangChain 适配器实例，用于后续的 LangChain 相关操作。"""
        from ..langchain_adapter import LangChainAdapter
        return LangChainAdapter(self)

    # === 核心服务接口 ===
    def list_services(self) -> List[ServiceInfo]:
        """
        列出服务列表（同步版本）
        - store上下文：聚合 main_client 下所有 client_id 的服务
        - agent上下文：聚合 agent_id 下所有 client_id 的服务
        """
        return self._sync_helper.run_async(self.list_services_async())

    async def list_services_async(self) -> List[ServiceInfo]:
        """
        列出服务列表（异步版本）
        - store上下文：聚合 main_client 下所有 client_id 的服务
        - agent上下文：聚合 agent_id 下所有 client_id 的服务
        """
        if self._context_type == ContextType.STORE:
            return await self._store.list_services()
        else:
            return await self._store.list_services(self._agent_id, agent_mode=True)

    def add_service(self, config: Union[ServiceConfigUnion, List[str], None] = None, json_file: str = None) -> 'MCPStoreContext':
        """
        增强版的服务添加方法（同步版本），支持多种配置格式

        Args:
            config: 服务配置，支持多种格式
            json_file: JSON文件路径，如果指定则读取该文件作为配置
        """
        return self._sync_helper.run_async(self.add_service_async(config, json_file), timeout=120.0)

    def add_service_with_details(self, config: Union[Dict[str, Any], List[Dict[str, Any]], str] = None) -> Dict[str, Any]:
        """
        添加服务并返回详细信息（同步版本）

        Args:
            config: 服务配置

        Returns:
            Dict: 包含添加结果的详细信息
        """
        return self._sync_helper.run_async(self.add_service_with_details_async(config), timeout=120.0)

    async def add_service_with_details_async(self, config: Union[Dict[str, Any], List[Dict[str, Any]], str] = None) -> Dict[str, Any]:
        """
        添加服务并返回详细信息（异步版本）

        Args:
            config: 服务配置

        Returns:
            Dict: 包含添加结果的详细信息
        """
        # 预处理配置
        try:
            processed_config = self._preprocess_service_config(config)
        except ValueError as e:
            return {
                "success": False,
                "added_services": [],
                "failed_services": self._extract_service_names(config),
                "service_details": {},
                "total_services": 0,
                "total_tools": 0,
                "message": str(e)
            }

        # 添加服务
        try:
            result = await self.add_service_async(processed_config)
        except Exception as e:
            return {
                "success": False,
                "added_services": [],
                "failed_services": self._extract_service_names(config),
                "service_details": {},
                "total_services": 0,
                "total_tools": 0,
                "message": f"Service addition failed: {str(e)}"
            }

        if result is None:
            return {
                "success": False,
                "added_services": [],
                "failed_services": self._extract_service_names(config),
                "service_details": {},
                "total_services": 0,
                "total_tools": 0,
                "message": "Service addition failed"
            }

        # 获取添加后的详情
        services = await self.list_services_async()
        tools = await self.list_tools_async()

        # 分析添加结果
        expected_service_names = self._extract_service_names(config)
        added_services = []
        service_details = {}

        for service_name in expected_service_names:
            service_info = next((s for s in services if getattr(s, "name", None) == service_name), None)
            if service_info:
                added_services.append(service_name)
                service_tools = [t for t in tools if getattr(t, "service_name", None) == service_name]
                service_details[service_name] = {
                    "tools_count": len(service_tools),
                    "status": getattr(service_info, "status", "unknown")
                }

        failed_services = [name for name in expected_service_names if name not in added_services]
        success = len(added_services) > 0
        total_tools = sum(details["tools_count"] for details in service_details.values())

        message = (
            f"Successfully added {len(added_services)} service(s) with {total_tools} tools"
            if success else
            f"Failed to add services. Available services: {[getattr(s, 'name', 'unknown') for s in services]}"
        )

        return {
            "success": success,
            "added_services": added_services,
            "failed_services": failed_services,
            "service_details": service_details,
            "total_services": len(added_services),
            "total_tools": total_tools,
            "message": message
        }

    def _preprocess_service_config(self, config: Union[Dict[str, Any], List[Dict[str, Any]], str] = None) -> Union[Dict[str, Any], List[Dict[str, Any]], str]:
        """预处理服务配置"""
        if not config:
            return config

        if isinstance(config, dict):
            # 处理单个服务配置
            if "mcpServers" in config:
                # mcpServers格式，直接返回
                return config
            else:
                # 单个服务格式，进行验证和转换
                processed = config.copy()

                # 验证必需字段
                if "name" not in processed:
                    raise ValueError("Service name is required")

                # 验证互斥字段
                if "url" in processed and "command" in processed:
                    raise ValueError("Cannot specify both url and command")

                # 自动推断transport类型
                if "url" in processed and "transport" not in processed:
                    url = processed["url"]
                    if "/sse" in url.lower():
                        processed["transport"] = "sse"
                    else:
                        processed["transport"] = "streamable-http"

                # 验证args格式
                if "command" in processed and not isinstance(processed.get("args", []), list):
                    raise ValueError("Args must be a list")

                return processed

        return config

    def _extract_service_names(self, config: Union[Dict[str, Any], List[Dict[str, Any]], str] = None) -> List[str]:
        """从配置中提取服务名称"""
        if not config:
            return []

        if isinstance(config, dict):
            if "name" in config:
                return [config["name"]]
            elif "mcpServers" in config:
                return list(config["mcpServers"].keys())
        elif isinstance(config, list):
            return config

        return []

    async def add_service_async(self, config: Union[ServiceConfigUnion, List[str], None] = None, json_file: str = None) -> 'MCPStoreContext':
        """
        增强版的服务添加方法，支持多种配置格式：
        1. URL方式：
           await add_service({
               "name": "weather",
               "url": "https://weather-api.example.com/mcp",
               "transport": "streamable-http"
           })

        2. 本地命令方式：
           await add_service({
               "name": "assistant",
               "command": "python",
               "args": ["./assistant_server.py"],
               "env": {"DEBUG": "true"}
           })

        3. MCPConfig字典方式：
           await add_service({
               "mcpServers": {
                   "weather": {
                       "url": "https://weather-api.example.com/mcp"
                   }
               }
           })

        4. 服务名称列表方式（从现有配置中选择）：
           await add_service(['weather', 'assistant'])

        5. 无参数方式（仅限Store上下文）：
           await add_service()  # 注册所有服务

        6. JSON文件方式：
           await add_service(json_file="path/to/config.json")  # 读取JSON文件作为配置

        所有新添加的服务都会同步到 mcp.json 配置文件中。

        Args:
            config: 服务配置，支持多种格式
            json_file: JSON文件路径，如果指定则读取该文件作为配置

        Returns:
            MCPStoreContext: 返回自身实例以支持链式调用
        """
        try:
            # 处理json_file参数
            if json_file is not None:
                logger.info(f"从JSON文件读取配置: {json_file}")
                try:
                    import json
                    import os

                    if not os.path.exists(json_file):
                        raise Exception(f"JSON文件不存在: {json_file}")

                    with open(json_file, 'r', encoding='utf-8') as f:
                        file_config = json.load(f)

                    logger.info(f"成功读取JSON文件，配置: {file_config}")

                    # 如果同时指定了config和json_file，优先使用json_file
                    if config is not None:
                        logger.warning("同时指定了config和json_file参数，将使用json_file")

                    config = file_config

                except Exception as e:
                    raise Exception(f"读取JSON文件失败: {e}")

            # 如果既没有config也没有json_file，且不是Store模式的全量注册，则报错
            if config is None and json_file is None and self._context_type != ContextType.STORE:
                raise Exception("必须指定config参数或json_file参数")

        except Exception as e:
            logger.error(f"参数处理失败: {e}")
            raise

        try:
            # 获取正确的 agent_id（Store级别使用main_client作为agent_id）
            agent_id = self._agent_id if self._context_type == ContextType.AGENT else self._store.orchestrator.client_manager.main_client_id
            logger.info(f"当前模式: {self._context_type.name}, agent_id: {agent_id}")
            
            # 处理不同的输入格式
            if config is None:
                # Store模式下的全量注册
                if self._context_type == ContextType.STORE:
                    logger.info("STORE模式-全量注册所有服务")
                    resp = await self._store.register_all_services_for_store()
                    logger.info(f"注册结果: {resp}")
                    if not (resp and resp.service_names):
                        raise Exception("服务注册失败")
                    # 无参数注册完成，直接返回
                    return self
                else:
                    logger.warning("AGENT模式-未指定服务配置")
                    raise Exception("AGENT模式必须指定服务配置")
                    
            # 处理列表格式
            elif isinstance(config, list):
                if not config:
                    raise Exception("列表为空")

                # 判断是服务名称列表还是服务配置列表
                if all(isinstance(item, str) for item in config):
                    # 服务名称列表
                    logger.info(f"注册指定服务: {config}")
                    if self._context_type == ContextType.STORE:
                        resp = await self._store.register_selected_services_for_store(config)
                    else:
                        resp = await self._store.register_services_for_agent(agent_id, config)
                    logger.info(f"注册结果: {resp}")
                    if not (resp and resp.service_names):
                        raise Exception("服务注册失败")
                    # 服务名称列表注册完成，直接返回
                    return self

                elif all(isinstance(item, dict) for item in config):
                    # 批量服务配置列表
                    logger.info(f"批量服务配置注册，数量: {len(config)}")

                    # 转换为MCPConfig格式
                    mcp_config = {"mcpServers": {}}
                    for service_config in config:
                        service_name = service_config.get("name")
                        if not service_name:
                            raise Exception("批量配置中的服务缺少name字段")
                        mcp_config["mcpServers"][service_name] = {
                            k: v for k, v in service_config.items() if k != "name"
                        }

                    # 将config设置为转换后的mcp_config，然后继续处理
                    config = mcp_config

                else:
                    raise Exception("列表中的元素类型不一致，必须全部是字符串（服务名称）或全部是字典（服务配置）")

            # 处理字典格式的配置（包括从批量配置转换来的）
            if isinstance(config, dict):
                # 转换为标准格式
                if "mcpServers" in config:
                    # 已经是MCPConfig格式
                    mcp_config = config
                else:
                    # 单个服务配置，需要转换为MCPConfig格式
                    service_name = config.get("name")
                    if not service_name:
                        raise Exception("服务配置缺少name字段")
                        
                    mcp_config = {
                        "mcpServers": {
                            service_name: {k: v for k, v in config.items() if k != "name"}
                        }
                    }
                
                # 更新配置文件和处理同名服务
                try:
                    # 1. 加载现有配置
                    current_config = self._store.config.load_config()

                    # 2. 合并新配置到mcp.json
                    for name, service_config in mcp_config["mcpServers"].items():
                        current_config["mcpServers"][name] = service_config

                    # 3. 保存更新后的配置
                    self._store.config.save_config(current_config)

                    # 4. 重新加载配置以确保同步
                    self._store.config.load_config()

                    # 5. 处理同名服务替换（新增逻辑）
                    created_client_ids = []
                    for name, service_config in mcp_config["mcpServers"].items():
                        # 使用新的同名服务处理逻辑
                        success = self._store.client_manager.replace_service_in_agent(
                            agent_id=agent_id,
                            service_name=name,
                            new_service_config=service_config
                        )
                        if not success:
                            raise Exception(f"替换服务 {name} 失败")
                        logger.info(f"成功处理同名服务: {name}")

                        # 获取刚创建的client_id用于Registry注册
                        client_ids = self._store.client_manager.get_agent_clients(agent_id)
                        for client_id in client_ids:
                            client_config = self._store.client_manager.get_client_config(client_id)
                            if client_config and name in client_config.get("mcpServers", {}):
                                if client_id not in created_client_ids:
                                    created_client_ids.append(client_id)
                                break

                    # 6. 注册服务到Registry（使用已创建的client配置）
                    logger.info(f"注册服务到Registry，使用client_ids: {created_client_ids}")
                    for client_id in created_client_ids:
                        client_config = self._store.client_manager.get_client_config(client_id)
                        if client_config:
                            try:
                                await self._store.orchestrator.register_json_services(client_config, client_id=client_id)
                                logger.info(f"成功注册client {client_id} 到Registry")
                            except Exception as e:
                                logger.warning(f"注册client {client_id} 到Registry失败: {e}")

                    logger.info(f"服务配置更新和Registry注册完成")

                except Exception as e:
                    raise Exception(f"更新配置文件失败: {e}")
            
            else:
                raise Exception(f"不支持的配置格式: {type(config)}")
            
            return self
            
        except Exception as e:
            logger.error(f"服务添加失败: {e}")
            raise

    def list_tools(self) -> List[ToolInfo]:
        """
        列出工具列表（同步版本）
        - store上下文：聚合 main_client 下所有 client_id 的工具
        - agent上下文：聚合 agent_id 下所有 client_id 的工具
        """
        return self._sync_helper.run_async(self.list_tools_async())

    async def list_tools_async(self) -> List[ToolInfo]:
        """
        列出工具列表（异步版本）
        - store上下文：聚合 main_client 下所有 client_id 的工具
        - agent上下文：聚合 agent_id 下所有 client_id 的工具
        """
        if self._context_type == ContextType.STORE:
            return await self._store.list_tools()
        else:
            return await self._store.list_tools(self._agent_id, agent_mode=True)

    def get_tools_with_stats(self) -> Dict[str, Any]:
        """
        获取工具列表及统计信息（同步版本）

        Returns:
            Dict: 包含工具列表和统计信息
        """
        return self._sync_helper.run_async(self.get_tools_with_stats_async())

    async def get_tools_with_stats_async(self) -> Dict[str, Any]:
        """
        获取工具列表及统计信息（异步版本）

        Returns:
            Dict: 包含工具列表和统计信息
        """
        tools = await self.list_tools_async()

        # 计算统计信息
        services_count = len(set(getattr(tool, "service_name", None) for tool in tools))

        return {
            "tools": tools,
            "metadata": {
                "total_tools": len(tools),
                "services_count": services_count,
                "context_type": self._context_type.name.lower(),
                "agent_id": self._agent_id if self._context_type == ContextType.AGENT else None,
                "last_updated": None  # 可以后续添加时间戳功能
            }
        }

    def get_system_stats(self) -> Dict[str, Any]:
        """
        获取系统统计信息（同步版本）

        Returns:
            Dict: 包含系统统计信息
        """
        return self._sync_helper.run_async(self.get_system_stats_async())

    async def get_system_stats_async(self) -> Dict[str, Any]:
        """
        获取系统统计信息（异步版本）

        Returns:
            Dict: 包含系统统计信息
        """
        # 获取基础数据
        services = await self.list_services_async()
        health_check = await self.check_services_async()
        tools = await self.list_tools_async()

        # 统计服务信息
        total_services = len(services) if services else 0
        healthy_services = 0
        unhealthy_services = 0

        if isinstance(health_check, dict) and "services" in health_check:
            for service in health_check["services"]:
                if service.get("status") == "healthy":
                    healthy_services += 1
                else:
                    unhealthy_services += 1

        total_tools = len(tools) if tools else 0

        # 按传输类型分组服务
        transport_stats = {}
        if services:
            for service in services:
                transport = getattr(service, 'transport_type', 'unknown')
                transport_name = transport.value if hasattr(transport, 'value') else str(transport)
                transport_stats[transport_name] = transport_stats.get(transport_name, 0) + 1

        return {
            "services": {
                "total": total_services,
                "healthy": healthy_services,
                "unhealthy": unhealthy_services,
                "by_transport": transport_stats
            },
            "tools": {
                "total": total_tools
            },
            "system": {
                "orchestrator_status": health_check.get("orchestrator_status", "unknown") if isinstance(health_check, dict) else "unknown",
                "context": self._context_type.name.lower(),
                "agent_id": self._agent_id if self._context_type == ContextType.AGENT else None
            }
        }

    def batch_add_services(self, services: List[Union[str, Dict[str, Any]]]) -> Dict[str, Any]:
        """
        批量添加服务（同步版本）

        Args:
            services: 服务列表，可以是服务名或配置字典

        Returns:
            Dict: 批量操作结果
        """
        return self._sync_helper.run_async(self.batch_add_services_async(services), timeout=180.0)

    async def batch_add_services_async(self, services: List[Union[str, Dict[str, Any]]]) -> Dict[str, Any]:
        """
        批量添加服务（异步版本）

        Args:
            services: 服务列表，可以是服务名或配置字典

        Returns:
            Dict: 批量操作结果
        """
        results = []

        for i, service in enumerate(services):
            try:
                if isinstance(service, str):
                    # 服务名方式
                    result = await self.add_service_async([service])
                elif isinstance(service, dict):
                    # 配置方式
                    result = await self.add_service_async(service)
                else:
                    results.append({
                        "index": i,
                        "success": False,
                        "message": "Invalid service format"
                    })
                    continue

                # add_service返回MCPStoreContext对象，表示成功
                success = result is not None
                results.append({
                    "index": i,
                    "service": service,
                    "success": success,
                    "message": f"Add operation {'succeeded' if success else 'failed'}"
                })

            except Exception as e:
                results.append({
                    "index": i,
                    "service": service,
                    "success": False,
                    "message": str(e)
                })

        success_count = sum(1 for r in results if r.get("success", False))
        total_count = len(results)

        return {
            "results": results,
            "summary": {
                "total": total_count,
                "succeeded": success_count,
                "failed": total_count - success_count
            },
            "success": success_count > 0,
            "message": f"Batch add completed: {success_count}/{total_count} succeeded"
        }

    def check_services(self) -> dict:
        """
        健康检查（同步版本），store/agent上下文自动判断
        - store上下文：聚合 main_client 下所有 client_id 的服务健康状态
        - agent上下文：聚合 agent_id 下所有 client_id 的服务健康状态
        """
        return self._sync_helper.run_async(self.check_services_async())

    async def check_services_async(self) -> dict:
        """
        异步健康检查，store/agent上下文自动判断
        - store上下文：聚合 main_client 下所有 client_id 的服务健康状态
        - agent上下文：聚合 agent_id 下所有 client_id 的服务健康状态
        """
        if self._context_type.name == 'STORE':
            return await self._store.get_health_status()
        elif self._context_type.name == 'AGENT':
            return await self._store.get_health_status(self._agent_id, agent_mode=True)
        else:
            print(f"[ERROR][check_services] 未知上下文类型: {self._context_type}")
            return {}

    def get_service_info(self, name: str) -> Any:
        """
        获取服务详情（同步版本），支持 store/agent 上下文
        - store上下文：在 main_client 下的所有 client 中查找服务
        - agent上下文：在指定 agent_id 下的所有 client 中查找服务
        """
        return self._sync_helper.run_async(self.get_service_info_async(name))

    async def get_service_info_async(self, name: str) -> Any:
        """
        获取服务详情（异步版本），支持 store/agent 上下文
        - store上下文：在 main_client 下的所有 client 中查找服务
        - agent上下文：在指定 agent_id 下的所有 client 中查找服务
        """
        if not name:
            return {}

        if self._context_type == ContextType.STORE:
            print(f"[INFO][get_service_info] STORE模式-在main_client中查找服务: {name}")
            return await self._store.get_service_info(name)
        elif self._context_type == ContextType.AGENT:
            print(f"[INFO][get_service_info] AGENT模式-在agent({self._agent_id})中查找服务: {name}")
            return await self._store.get_service_info(name, self._agent_id)
        else:
            print(f"[ERROR][get_service_info] 未知上下文类型: {self._context_type}")
            return {}

    def use_tool(self, tool_name: str, args: Union[Dict[str, Any], str] = None, **kwargs) -> Any:
        """
        使用工具（同步版本），支持 store/agent 上下文

        用户友好的工具调用接口，支持多种工具名称格式：
        - 直接工具名: "get_weather"
        - 服务前缀: "weather__get_weather"
        - 旧格式: "weather_get_weather"

        Args:
            tool_name: 工具名称（支持多种格式）
            args: 工具参数（字典或JSON字符串）
            **kwargs: 额外参数（timeout, progress_handler等）

        Returns:
            Any: 工具执行结果（FastMCP 标准格式）
        """
        return self._sync_helper.run_async(self.use_tool_async(tool_name, args, **kwargs))

    def to_langchain_tools(self):
        """
        将 MCPStore 工具转换为 LangChain 工具（同步版本）

        Returns:
            List[Tool]: LangChain 工具列表
        """
        return self._sync_helper.run_async(self.to_langchain_tools_async())

    async def to_langchain_tools_async(self):
        """
        将 MCPStore 工具转换为 LangChain 工具（异步版本）

        Returns:
            List[Tool]: LangChain 工具列表
        """
        try:
            from mcpstore.adapters.langchain_adapter import LangChainAdapter
            adapter = LangChainAdapter(self)
            return await adapter.list_tools_async()
        except ImportError:
            raise ImportError("需要安装 langchain 依赖: pip install langchain langchain-core")

    async def use_tool_async(self, tool_name: str, args: Dict[str, Any] = None, **kwargs) -> Any:
        """
        使用工具（异步版本），支持 store/agent 上下文

        Args:
            tool_name: 工具名称（支持多种格式）
            args: 工具参数
            **kwargs: 额外参数（timeout, progress_handler等）

        Returns:
            Any: 工具执行结果（FastMCP 标准格式）
        """
        args = args or {}

        # 获取可用工具列表用于智能解析
        available_tools = []
        try:
            if self._context_type == ContextType.STORE:
                tools = await self._store.list_tools()
            else:
                tools = await self._store.list_tools(self._agent_id, agent_mode=True)

            # 构建工具信息，包含显示名称和原始名称
            for tool in tools:
                # 现在 tool.name 就是显示名称
                display_name = tool.name
                original_name = self._extract_original_tool_name(display_name, tool.service_name)

                available_tools.append({
                    "name": display_name,           # 显示名称（如：mcpstore-demo-weather_get_current_weather）
                    "original_name": original_name, # 原始名称（如：get_current_weather）
                    "service_name": tool.service_name
                })

            logger.debug(f"Available tools for resolution: {len(available_tools)}")
        except Exception as e:
            logger.warning(f"Failed to get available tools for resolution: {e}")

        # 使用统一解析器解析工具名称
        from mcpstore.core.tool_resolver import ToolNameResolver

        resolver = ToolNameResolver(available_services=self._get_available_services())

        try:
            resolution = resolver.resolve_tool_name(tool_name, available_tools)
            logger.debug(f"Tool resolved: {tool_name} -> {resolution.service_name}::{resolution.original_tool_name} ({resolution.resolution_method})")
        except ValueError as e:
            raise ValueError(f"Tool resolution failed: {e}")

        # 构造标准化的工具执行请求
        if self._context_type == ContextType.STORE:
            logger.info(f"[STORE] Executing tool: {resolution.original_tool_name} from service: {resolution.service_name}")
            request = ToolExecutionRequest(
                tool_name=resolution.original_tool_name,
                service_name=resolution.service_name,
                args=args,
                **kwargs
            )
        else:
            logger.info(f"[AGENT:{self._agent_id}] Executing tool: {resolution.original_tool_name} from service: {resolution.service_name}")
            request = ToolExecutionRequest(
                tool_name=resolution.original_tool_name,
                service_name=resolution.service_name,
                args=args,
                agent_id=self._agent_id,
                **kwargs
            )

        return await self._store.process_tool_request(request)

    def _get_available_services(self) -> List[str]:
        """获取可用服务列表"""
        try:
            if self._context_type == ContextType.STORE:
                services = self._store.for_store().list_services()
            else:
                services = self._store.for_agent(self._agent_id).list_services()
            return [service.name for service in services]
        except Exception:
            return []

    def _extract_original_tool_name(self, display_name: str, service_name: str) -> str:
        """
        从显示名称中提取原始工具名称

        Args:
            display_name: 显示名称（如：mcpstore-demo-weather_get_current_weather）
            service_name: 服务名称（如：mcpstore-demo-weather）

        Returns:
            原始工具名称（如：get_current_weather）
        """
        # 尝试移除服务名前缀
        if display_name.startswith(f"{service_name}_"):
            return display_name[len(service_name) + 1:]

        # 如果没有前缀，可能就是原始名称
        return display_name

    # === 上下文信息 ===
    @property
    def context_type(self) -> ContextType:
        """获取上下文类型"""
        return self._context_type

    @property
    def agent_id(self) -> Optional[str]:
        """获取当前agent_id"""
        return self._agent_id 

    def show_mcpconfig(self) -> Dict[str, Any]:
        """
        根据当前上下文（store/agent）获取对应的配置信息
        
        Returns:
            Dict[str, Any]: 包含所有相关client配置的字典
        """
        # 获取所有相关的client_ids
        agent_id = self._agent_id if self._context_type == ContextType.AGENT else self._store.orchestrator.client_manager.main_client_id
        client_ids = self._store.orchestrator.client_manager.get_agent_clients(agent_id)
        
        # 获取每个client的配置
        result = {}
        for client_id in client_ids:
            client_config = self._store.orchestrator.client_manager.get_client_config(client_id)
            if client_config:
                result[client_id] = client_config
                
        return result 

    def update_service(self, name: str, config: Dict[str, Any]) -> bool:
        """
        更新服务配置（同步版本）- 完全替换配置

        Args:
            name: 服务名称（不可更改）
            config: 新的完整服务配置（必须包含url或command字段）

        Returns:
            bool: 更新是否成功

        Note:
            此方法会完全替换服务配置。如需增量更新，请使用 patch_service() 方法。
        """
        return self._sync_helper.run_async(self.update_service_async(name, config))

    def patch_service(self, name: str, updates: Dict[str, Any]) -> bool:
        """
        增量更新服务配置（同步版本）- 推荐使用

        Args:
            name: 服务名称（不可更改）
            updates: 要更新的字段（会与现有配置合并）

        Returns:
            bool: 更新是否成功
        """
        return self._sync_helper.run_async(self.patch_service_async(name, updates))

    async def update_service_async(self, name: str, config: Dict[str, Any]) -> bool:
        """
        更新服务配置
        
        Args:
            name: 服务名称（不可更改）
            config: 新的服务配置
            
        Returns:
            bool: 更新是否成功
            
        Raises:
            ServiceNotFoundError: 服务不存在
            InvalidConfigError: 配置无效
        """
        try:
            # 1. 验证服务是否存在
            if not self._store.config.get_service_config(name):
                raise ServiceNotFoundError(f"Service {name} not found")
            
            # 2. 更新 mcp.json 中的配置（无论是 store 还是 agent 级别都要更新）
            if not self._store.config.update_service(name, config):
                raise InvalidConfigError(f"Failed to update service {name}")
            
            # 3. 获取需要更新的 client_ids
            if self._context_type == ContextType.STORE:
                # store 级别：更新所有 client
                client_ids = self._store.orchestrator.client_manager.get_main_client_ids()
            else:
                # agent 级别：同样更新所有配置
                client_ids = self._store.orchestrator.client_manager.get_main_client_ids()
            
            # 4. 更新每个 client 的配置
            for client_id in client_ids:
                client_config = self._store.orchestrator.client_manager.get_client_config(client_id)
                if client_config and name in client_config.get("mcpServers", {}):
                    client_config["mcpServers"][name] = config
                    self._store.orchestrator.client_manager.save_client_config(client_id, client_config)
            
            return True
            
        except Exception as e:
            logging.error(f"Failed to update service {name}: {str(e)}")
            raise

    async def patch_service_async(self, name: str, updates: Dict[str, Any]) -> bool:
        """
        增量更新服务配置（异步版本）

        Args:
            name: 服务名称（不可更改）
            updates: 要更新的字段（会与现有配置合并）

        Returns:
            bool: 更新是否成功

        Raises:
            ServiceNotFoundError: 服务不存在
            InvalidConfigError: 配置无效
        """
        try:
            # 1. 获取当前服务配置
            current_config = self._store.config.get_service_config(name)
            if not current_config:
                raise ServiceNotFoundError(f"Service {name} not found")

            # 2. 合并配置（updates 覆盖 current_config）
            merged_config = {**current_config, **updates}

            # 3. 调用完整更新方法
            return await self.update_service_async(name, merged_config)

        except Exception as e:
            logging.error(f"Failed to patch service {name}: {str(e)}")
            raise

    def delete_service(self, name: str) -> bool:
        """
        删除服务（同步版本）

        Args:
            name: 要删除的服务名称

        Returns:
            bool: 删除是否成功
        """
        return self._sync_helper.run_async(self.delete_service_async(name))

    async def delete_service_async(self, name: str) -> bool:
        """
        删除服务
        
        Args:
            name: 要删除的服务名称
            
        Returns:
            bool: 删除是否成功
            
        Raises:
            ServiceNotFoundError: 服务不存在
            DeleteServiceError: 删除失败
        """
        try:
            # 1. 验证服务是否存在
            if not self._store.config.get_service_config(name):
                raise ServiceNotFoundError(f"Service {name} not found")
            
            # 2. 根据上下文确定删除范围
            if self._context_type == ContextType.STORE:
                # store 级别：删除所有 client 中的服务并更新 mcp.json
                client_ids = self._store.orchestrator.client_manager.get_main_client_ids()
                
                # 从 mcp.json 中删除
                if not self._store.config.remove_service(name):
                    raise DeleteServiceError(f"Failed to remove service {name} from mcp.json")
                
                # 从所有 client 配置中删除
                for client_id in client_ids:
                    client_config = self._store.orchestrator.client_manager.get_client_config(client_id)
                    if client_config and name in client_config.get("mcpServers", {}):
                        del client_config["mcpServers"][name]
                        self._store.orchestrator.client_manager.save_client_config(client_id, client_config)
                
            else:
                # agent 级别：只删除该 agent 的 client 列表中的服务
                client_ids = self._store.orchestrator.client_manager.get_agent_clients(self._agent_id)
                
                # 从指定 agent 的 client 配置中删除
                for client_id in client_ids:
                    client_config = self._store.orchestrator.client_manager.get_client_config(client_id)
                    if client_config and name in client_config.get("mcpServers", {}):
                        del client_config["mcpServers"][name]
                        self._store.orchestrator.client_manager.save_client_config(client_id, client_config)
            
            return True
            
        except Exception as e:
            logging.error(f"Failed to delete service {name}: {str(e)}")
            raise

    def for_langchain(self) -> 'LangChainAdapter':
        """返回LangChain适配器实例"""
        from mcpstore.adapters.langchain_adapter import LangChainAdapter
        return LangChainAdapter(self)

    def reset_config(self) -> bool:
        """重置配置（同步版本）"""
        return self._sync_helper.run_async(self.reset_config_async(), timeout=60.0)

    async def reset_config_async(self) -> bool:
        """
        重置配置（链式操作）
        - Store级别：重置main_client的配置，并从文件中删除相关配置
        - Agent级别：重置指定Agent的配置，并从文件中删除相关配置

        Returns:
            是否成功重置
        """
        try:
            if self._agent_id is None:
                # Store级别重置
                main_client_id = self._store.orchestrator.client_manager.main_client_id

                # 1. 清理registry中的store级别数据
                if main_client_id in self._store.orchestrator.registry.sessions:
                    del self._store.orchestrator.registry.sessions[main_client_id]
                if main_client_id in self._store.orchestrator.registry.service_health:
                    del self._store.orchestrator.registry.service_health[main_client_id]
                if main_client_id in self._store.orchestrator.registry.tool_cache:
                    del self._store.orchestrator.registry.tool_cache[main_client_id]
                if main_client_id in self._store.orchestrator.registry.tool_to_session_map:
                    del self._store.orchestrator.registry.tool_to_session_map[main_client_id]

                # 2. 清理重连队列
                self._cleanup_reconnection_queue_for_client(main_client_id)

                # 3. 从文件中删除Store相关配置
                file_success = self._store.orchestrator.client_manager.remove_store_from_files(main_client_id)

                if file_success:
                    logging.info("Successfully reset store config, registry and files")
                else:
                    logging.warning("Reset store config and registry, but failed to clean files")

                return file_success
            else:
                # Agent级别重置

                # 1. 清理registry中的agent级别数据
                if self._agent_id in self._store.orchestrator.registry.sessions:
                    del self._store.orchestrator.registry.sessions[self._agent_id]
                if self._agent_id in self._store.orchestrator.registry.service_health:
                    del self._store.orchestrator.registry.service_health[self._agent_id]
                if self._agent_id in self._store.orchestrator.registry.tool_cache:
                    del self._store.orchestrator.registry.tool_cache[self._agent_id]
                if self._agent_id in self._store.orchestrator.registry.tool_to_session_map:
                    del self._store.orchestrator.registry.tool_to_session_map[self._agent_id]

                # 2. 清理重连队列
                agent_clients = self._store.orchestrator.client_manager.get_agent_clients(self._agent_id)
                for client_id in agent_clients:
                    self._cleanup_reconnection_queue_for_client(client_id)

                # 3. 从文件中删除Agent相关配置
                file_success = self._store.orchestrator.client_manager.remove_agent_from_files(self._agent_id)

                if file_success:
                    logging.info(f"Successfully reset agent {self._agent_id} config, registry and files")
                else:
                    logging.warning(f"Reset agent {self._agent_id} config and registry, but failed to clean files")

                return file_success

        except Exception as e:
            logging.error(f"Failed to reset config: {str(e)}")
            return False

    def _cleanup_reconnection_queue_for_client(self, client_id: str):
        """清理重连队列中与指定client相关的条目"""
        try:
            # 查找所有与该client相关的重连条目
            entries_to_remove = []
            for service_key in self._store.orchestrator.smart_reconnection.entries:
                if service_key.startswith(f"{client_id}:"):
                    entries_to_remove.append(service_key)

            # 移除这些条目
            for entry in entries_to_remove:
                self._store.orchestrator.smart_reconnection.remove_service(entry)

            if entries_to_remove:
                logging.info(f"Cleaned up {len(entries_to_remove)} reconnection queue entries for client {client_id}")

        except Exception as e:
            logging.warning(f"Failed to cleanup reconnection queue for client {client_id}: {e}")

    def show_mcpconfig(self) -> dict:
        """显示MCP配置"""
        try:
            config = self._store.config.load_config()
            # 确保返回格式正确
            if isinstance(config, dict) and 'mcpServers' in config:
                return config
            else:
                logging.warning("Invalid MCP config format")
                return {"mcpServers": {}}
        except Exception as e:
            logging.error(f"Failed to show MCP config: {e}")
            return {"mcpServers": {}}

    def get_service_status(self, name: str) -> dict:
        """获取单个服务的状态信息（同步版本）"""
        return self._sync_helper.run_async(self.get_service_status_async(name))

    async def get_service_status_async(self, name: str) -> dict:
        """获取单个服务的状态信息"""
        try:
            service_info = await self.get_service_info_async(name)
            if hasattr(service_info, 'service') and service_info.service:
                return {
                    "name": service_info.service.name,
                    "status": service_info.service.status,
                    "connected": service_info.connected,
                    "tool_count": service_info.service.tool_count,
                    "last_heartbeat": service_info.service.last_heartbeat,
                    "transport_type": service_info.service.transport_type
                }
            else:
                return {
                    "name": name,
                    "status": "not_found",
                    "connected": False,
                    "tool_count": 0,
                    "last_heartbeat": None,
                    "transport_type": None
                }
        except Exception as e:
            logging.error(f"Failed to get service status for {name}: {e}")
            return {
                "name": name,
                "status": "error",
                "connected": False,
                "error": str(e)
            }

    def restart_service(self, name: str) -> bool:
        """重启指定服务（同步版本）"""
        return self._sync_helper.run_async(self.restart_service_async(name))

    async def restart_service_async(self, name: str) -> bool:
        """重启指定服务"""
        try:
            # 首先验证服务是否存在
            service_info = await self.get_service_info_async(name)
            if not service_info or not (hasattr(service_info, 'service') and service_info.service):
                logging.error(f"Service {name} not found in registry")
                return False

            # 获取服务配置
            service_config = self._store.config.get_service_config(name)
            if not service_config:
                logging.error(f"Service config not found for {name} in mcp.json")
                # 尝试从当前运行的服务中获取配置信息
                logging.info(f"Attempting to restart service {name} without config reload")
                # 简单的重连尝试
                try:
                    # 获取当前上下文的client_id
                    agent_id = self._agent_id if self._context_type == ContextType.AGENT else self._store.orchestrator.client_manager.main_client_id
                    client_ids = self._store.orchestrator.client_manager.get_agent_clients(agent_id)

                    for client_id in client_ids:
                        if self._store.orchestrator.registry.has_service(client_id, name):
                            # 尝试重新连接服务
                            success, message = await self._store.orchestrator.connect_service(name)
                            if success:
                                logging.info(f"Service {name} reconnected successfully")
                                return True

                    logging.error(f"Failed to reconnect service {name}")
                    return False
                except Exception as e:
                    logging.error(f"Failed to reconnect service {name}: {e}")
                    return False

            # 先删除服务
            delete_success = await self.delete_service_async(name)
            if not delete_success:
                logging.warning(f"Failed to delete service {name} during restart, attempting to continue")

            # 等待一小段时间确保服务完全停止
            import asyncio
            await asyncio.sleep(1)

            # 构造添加服务的配置
            add_config = {
                "name": name,
                **service_config
            }

            # 重新添加服务
            await self.add_service_async(add_config)
            logging.info(f"Service {name} restarted successfully")
            return True

        except Exception as e:
            logging.error(f"Failed to restart service {name}: {e}")
            return False



    # === 文件直接重置功能 ===
    def reset_mcp_json_file(self) -> bool:
        """直接重置MCP JSON配置文件（同步版本）"""
        return self._sync_helper.run_async(self.reset_mcp_json_file_async(), timeout=60.0)

    async def reset_mcp_json_file_async(self) -> bool:
        """
        直接重置MCP JSON配置文件（仅Store级别可用）
        备份后重置为空字典 {"mcpServers": {}}

        Returns:
            是否成功重置
        """
        if self._agent_id is not None:
            logging.warning("reset_mcp_json_file is only available for store level")
            return False

        try:
            success = self._store.config.reset_mcp_json_file()
            if success:
                # 重置后需要重新加载配置
                await self._store.orchestrator.setup()
                logging.info("Successfully reset MCP JSON file and reloaded")
            return success

        except Exception as e:
            logging.error(f"Failed to reset MCP JSON file: {str(e)}")
            return False

    def reset_client_services_file(self) -> bool:
        """直接重置client_services.json文件（同步版本）"""
        return self._sync_helper.run_async(self.reset_client_services_file_async(), timeout=60.0)

    async def reset_client_services_file_async(self) -> bool:
        """
        直接重置client_services.json文件（仅Store级别可用）
        备份后重置为空字典 {}

        Returns:
            是否成功重置
        """
        if self._agent_id is not None:
            logging.warning("reset_client_services_file is only available for store level")
            return False

        try:
            success = self._store.orchestrator.client_manager.reset_client_services_file()
            if success:
                # 重置后需要重新加载配置
                await self._store.orchestrator.setup()
                logging.info("Successfully reset client_services.json file and reloaded")
            return success

        except Exception as e:
            logging.error(f"Failed to reset client_services.json file: {str(e)}")
            return False

    def reset_agent_clients_file(self) -> bool:
        """直接重置agent_clients.json文件（同步版本）"""
        return self._sync_helper.run_async(self.reset_agent_clients_file_async(), timeout=60.0)

    async def reset_agent_clients_file_async(self) -> bool:
        """
        直接重置agent_clients.json文件（仅Store级别可用）
        备份后重置为空字典 {}

        Returns:
            是否成功重置
        """
        if self._agent_id is not None:
            logging.warning("reset_agent_clients_file is only available for store level")
            return False

        try:
            success = self._store.orchestrator.client_manager.reset_agent_clients_file()
            if success:
                # 重置后需要重新加载配置
                await self._store.orchestrator.setup()
                logging.info("Successfully reset agent_clients.json file and reloaded")
            return success

        except Exception as e:
            logging.error(f"Failed to reset agent_clients.json file: {str(e)}")
            return False



    def get_unified_config(self) -> 'UnifiedConfigManager':
        """获取统一配置管理器

        Returns:
            UnifiedConfigManager: 统一配置管理器实例
        """
        return self._store.get_unified_config()

    # === 新功能：工具转换 ===

    def create_simple_tool(self, original_tool: str, friendly_name: str = None) -> 'MCPStoreContext':
        """
        创建简化版工具

        Args:
            original_tool: 原始工具名
            friendly_name: 友好名称（可选）

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            result = self._transformation_manager.create_simple_weather_tool(original_tool)
            logging.info(f"[{self._context_type.value}] Created simple tool for: {original_tool}")
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to create simple tool: {e}")
            return self

    def create_safe_tool(self, original_tool: str, validation_rules: Dict[str, Any]) -> 'MCPStoreContext':
        """
        创建安全版工具（带验证）

        Args:
            original_tool: 原始工具名
            validation_rules: 验证规则字典

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            # 转换验证规则为函数
            validation_functions = {}
            for param, rule in validation_rules.items():
                if isinstance(rule, dict):
                    validation_functions[param] = self._create_validation_function(rule)

            result = self._transformation_manager.transformer.create_validated_tool(
                original_tool, validation_functions
            )
            logging.info(f"[{self._context_type.value}] Created safe tool for: {original_tool}")
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to create safe tool: {e}")
            return self

    # === 新功能：环境管理 ===

    def switch_environment(self, environment: str) -> 'MCPStoreContext':
        """
        切换运行环境

        Args:
            environment: 环境名称 (development, testing, production)

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            success = self._component_manager.switch_environment(environment)
            if success:
                logging.info(f"[{self._context_type.value}] Switched to environment: {environment}")
            else:
                logging.warning(f"[{self._context_type.value}] Failed to switch to environment: {environment}")
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Error switching environment: {e}")
            return self

    def create_custom_environment(self, name: str, allowed_categories: List[str]) -> 'MCPStoreContext':
        """
        创建自定义环境

        Args:
            name: 环境名称
            allowed_categories: 允许的工具分类

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            self._component_manager.create_custom_environment(name, allowed_categories)
            logging.info(f"[{self._context_type.value}] Created custom environment: {name}")
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to create environment {name}: {e}")
            return self

    # === 新功能：OpenAPI 集成 ===

    async def import_api_async(self, api_url: str, api_name: str = None) -> 'MCPStoreContext':
        """
        导入 OpenAPI 服务（异步）

        Args:
            api_url: API 规范 URL
            api_name: API 名称（可选）

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            import time
            api_name = api_name or f"api_{int(time.time())}"
            result = await self._openapi_manager.import_openapi_service(
                name=api_name,
                spec_url=api_url
            )
            logging.info(f"[{self._context_type.value}] Imported API {api_name}: {result.get('total_endpoints', 0)} endpoints")
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to import API {api_url}: {e}")
            return self

    def import_api(self, api_url: str, api_name: str = None) -> 'MCPStoreContext':
        """
        导入 OpenAPI 服务（同步）

        Args:
            api_url: API 规范 URL
            api_name: API 名称（可选）

        Returns:
            MCPStoreContext: 支持链式调用
        """
        return self._sync_helper.run_async(self.import_api_async(api_url, api_name))

    # === 新功能：性能优化 ===

    def enable_caching(self, patterns: Dict[str, int] = None) -> 'MCPStoreContext':
        """
        启用智能缓存

        Args:
            patterns: 缓存模式配置 {工具模式: TTL秒数}

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            self._performance_optimizer.setup_tool_caching(patterns)
            logging.info(f"[{self._context_type.value}] Enabled intelligent caching")
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to enable caching: {e}")
            return self

    def get_performance_report(self) -> Dict[str, Any]:
        """
        获取性能报告

        Returns:
            Dict: 性能报告数据
        """
        try:
            return self._performance_optimizer.get_performance_summary()
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to get performance report: {e}")
            return {}

    # === 新功能：认证安全 ===

    def setup_auth(self, auth_type: str = "bearer", enabled: bool = True) -> 'MCPStoreContext':
        """
        设置认证

        Args:
            auth_type: 认证类型 ("bearer", "api_key")
            enabled: 是否启用

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            if auth_type == "bearer":
                self._auth_manager.setup_bearer_auth(enabled)
            elif auth_type == "api_key":
                self._auth_manager.setup_api_key_auth(enabled)
            else:
                logging.warning(f"[{self._context_type.value}] Unknown auth type: {auth_type}")
                return self

            logging.info(f"[{self._context_type.value}] Setup {auth_type} authentication: {'enabled' if enabled else 'disabled'}")
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to setup authentication: {e}")
            return self

    # === 新功能：监控分析 ===

    def get_usage_stats(self) -> Dict[str, Any]:
        """
        获取使用统计

        Returns:
            Dict: 使用统计数据
        """
        try:
            return self._monitoring_manager.get_dashboard_data()
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to get usage stats: {e}")
            return {}

    def record_tool_execution(self, tool_name: str, duration: float, success: bool, error: Exception = None) -> 'MCPStoreContext':
        """
        记录工具执行情况

        Args:
            tool_name: 工具名称
            duration: 执行时间
            success: 是否成功
            error: 错误信息（可选）

        Returns:
            MCPStoreContext: 支持链式调用
        """
        try:
            service_name = self._extract_service_name(tool_name)
            self._monitoring_manager.record_tool_execution(
                tool_name=tool_name,
                service_name=service_name,
                duration=duration,
                success=success,
                user_id=self._agent_id,
                error=error
            )
            return self
        except Exception as e:
            logging.error(f"[{self._context_type.value}] Failed to record tool execution: {e}")
            return self

    # === 辅助方法 ===

    def _create_validation_function(self, rule: Dict[str, Any]) -> callable:
        """创建验证函数"""
        def validate(value):
            if "min_length" in rule and len(str(value)) < rule["min_length"]:
                raise ValueError(f"Value too short, minimum length: {rule['min_length']}")
            if "max_length" in rule and len(str(value)) > rule["max_length"]:
                raise ValueError(f"Value too long, maximum length: {rule['max_length']}")
            if "pattern" in rule:
                import re
                if not re.match(rule["pattern"], str(value)):
                    raise ValueError(f"Value doesn't match pattern: {rule['pattern']}")
            return value
        return validate

    def _extract_service_name(self, tool_name: str) -> str:
        """从工具名提取服务名"""
        if "_" in tool_name:
            return tool_name.split("_")[0]
        return "unknown"

    # === 监控和统计接口 ===

    def get_performance_metrics(self) -> PerformanceMetrics:
        """获取性能指标"""
        return self._monitoring.get_performance_metrics()

    async def get_performance_metrics_async(self) -> PerformanceMetrics:
        """异步获取性能指标"""
        return self.get_performance_metrics()

    def get_tool_usage_stats(self, limit: int = 10) -> List[ToolUsageStats]:
        """获取工具使用统计"""
        return self._monitoring.get_tool_usage_stats(limit)

    async def get_tool_usage_stats_async(self, limit: int = 10) -> List[ToolUsageStats]:
        """异步获取工具使用统计"""
        return self.get_tool_usage_stats(limit)

    def get_alerts(self, unresolved_only: bool = False) -> List[AlertInfo]:
        """获取告警列表"""
        return self._monitoring.get_alerts(unresolved_only)

    async def get_alerts_async(self, unresolved_only: bool = False) -> List[AlertInfo]:
        """异步获取告警列表"""
        return self.get_alerts(unresolved_only)

    def add_alert(self, alert_type: str, title: str, message: str,
                  service_name: Optional[str] = None) -> str:
        """添加告警"""
        return self._monitoring.add_alert(alert_type, title, message, service_name)

    async def add_alert_async(self, alert_type: str, title: str, message: str,
                             service_name: Optional[str] = None) -> str:
        """异步添加告警"""
        return self.add_alert(alert_type, title, message, service_name)

    def resolve_alert(self, alert_id: str) -> bool:
        """解决告警"""
        return self._monitoring.resolve_alert(alert_id)

    async def resolve_alert_async(self, alert_id: str) -> bool:
        """异步解决告警"""
        return self.resolve_alert(alert_id)

    def clear_all_alerts(self) -> bool:
        """清除所有告警"""
        return self._monitoring.clear_all_alerts()

    async def clear_all_alerts_async(self) -> bool:
        """异步清除所有告警"""
        return self.clear_all_alerts()

    async def check_network_endpoints(self, endpoints: List[Dict[str, str]]) -> List[NetworkEndpoint]:
        """检查网络端点状态"""
        return await self._monitoring.check_network_endpoints(endpoints)

    def get_system_resource_info(self) -> SystemResourceInfo:
        """获取系统资源信息"""
        return self._monitoring.get_system_resource_info()

    async def get_system_resource_info_async(self) -> SystemResourceInfo:
        """异步获取系统资源信息"""
        return self.get_system_resource_info()

    def record_api_call(self, response_time: float):
        """记录API调用"""
        self._monitoring.record_api_call(response_time)

    def record_tool_execution(self, tool_name: str, service_name: str,
                            response_time: float, success: bool):
        """记录工具执行"""
        self._monitoring.record_tool_execution(tool_name, service_name, response_time, success)

    def increment_active_connections(self):
        """增加活跃连接数"""
        self._monitoring.increment_active_connections()

    def decrement_active_connections(self):
        """减少活跃连接数"""
        self._monitoring.decrement_active_connections()





from typing import Dict, Any, Optional, Set
from datetime import datetime, timedelta
import uuid
import logging
from fastmcp import Client

logger = logging.getLogger(__name__)

class AgentSession:
    """Agent 会话类"""
    def __init__(self, agent_id: str):
        self.agent_id = agent_id
        self.services: Dict[str, Client] = {}  # service_name -> Client
        self.tools: Dict[str, Dict[str, Any]] = {}  # tool_name -> tool_info
        self.last_active = datetime.now()
        self.created_at = datetime.now()
        
    def update_activity(self):
        """更新最后活动时间"""
        self.last_active = datetime.now()
        
    def add_service(self, service_name: str, client: Client):
        """添加服务"""
        self.services[service_name] = client
        
    def add_tool(self, tool_name: str, tool_info: Dict[str, Any], service_name: str):
        """添加工具"""
        self.tools[tool_name] = {
            **tool_info,
            "service_name": service_name
        }
        
    def get_service_for_tool(self, tool_name: str) -> Optional[str]:
        """获取工具对应的服务名"""
        return self.tools.get(tool_name, {}).get("service_name")
        
    def get_all_tools(self) -> Dict[str, Dict[str, Any]]:
        """获取所有工具信息"""
        return self.tools

class SessionManager:
    """会话管理器"""
    def __init__(self, session_timeout: int = 3600):
        self.sessions: Dict[str, AgentSession] = {}
        self.session_timeout = timedelta(seconds=session_timeout)
        
    def create_session(self, agent_id: Optional[str] = None) -> AgentSession:
        """创建新会话"""
        if not agent_id:
            agent_id = str(uuid.uuid4())
            
        session = AgentSession(agent_id)
        self.sessions[agent_id] = session
        logger.info(f"Created new session for agent {agent_id}")
        return session
        
    def get_session(self, agent_id: str) -> Optional[AgentSession]:
        """获取会话"""
        session = self.sessions.get(agent_id)
        if session:
            # 检查会话是否过期
            if datetime.now() - session.last_active > self.session_timeout:
                logger.info(f"Session expired for agent {agent_id}")
                del self.sessions[agent_id]
                return None
            session.update_activity()
        return session
        
    def get_or_create_session(self, agent_id: Optional[str] = None) -> AgentSession:
        """获取或创建会话"""
        if agent_id and (session := self.get_session(agent_id)):
            return session
        return self.create_session(agent_id)
        
    def cleanup_expired_sessions(self):
        """清理过期会话"""
        now = datetime.now()
        expired = [
            agent_id for agent_id, session in self.sessions.items()
            if now - session.last_active > self.session_timeout
        ]
        for agent_id in expired:
            del self.sessions[agent_id]
            logger.info(f"Cleaned up expired session for agent {agent_id}") 



#!/usr/bin/env python3
"""
MCPStore 独立配置系统
完全不依赖环境变量，通过默认参数和初始化配置工作
"""

import logging
from typing import Dict, Any, Optional, List, Union
from pathlib import Path
from dataclasses import dataclass, field
from copy import deepcopy

logger = logging.getLogger(__name__)

@dataclass
class StandaloneConfig:
    """独立配置类 - 不依赖任何环境变量"""
    
    # === 核心配置 ===
    heartbeat_interval_seconds: int = 60
    http_timeout_seconds: int = 30
    reconnection_interval_seconds: int = 300
    cleanup_interval_seconds: int = 3600
    
    # === 网络配置 ===
    streamable_http_endpoint: str = "/mcp"
    default_transport: str = "http"
    
    # === 文件路径配置 ===
    config_dir: Optional[str] = None  # 如果为None，使用内存配置
    mcp_config_file: Optional[str] = None
    client_services_file: Optional[str] = None
    agent_clients_file: Optional[str] = None
    
    # === 服务配置 ===
    known_services: Dict[str, Dict[str, Any]] = field(default_factory=lambda: {
        "mcpstore-wiki": {
            "url": "http://mcpstore.wiki/mcp",
            "transport": "http",
            "description": "MCPStore Wiki Service"
        },
        "howtocook": {
            "command": "npx",
            "args": ["-y", "howtocook-mcp"],
            "description": "How to Cook MCP Service"
        }
    })
    
    # === 环境隔离配置 ===
    isolated_environment: bool = True  # 是否使用隔离环境
    base_environment: Dict[str, str] = field(default_factory=lambda: {
        "PYTHONPATH": ".",
        "PATH": "/usr/local/bin:/usr/bin:/bin"  # 基础PATH，不依赖系统
    })
    
    # === 日志配置 ===
    log_level: str = "INFO"
    log_format: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    enable_debug: bool = False

class StandaloneConfigManager:
    """独立配置管理器 - 完全不依赖环境变量"""
    
    def __init__(self, config: Optional[StandaloneConfig] = None):
        """
        初始化独立配置管理器
        
        Args:
            config: 自定义配置，如果为None则使用默认配置
        """
        self.config = config or StandaloneConfig()
        self._runtime_config: Dict[str, Any] = {}
        self._service_configs: Dict[str, Dict[str, Any]] = {}
        
        # 初始化默认配置
        self._initialize_default_configs()
        
        logger.info("StandaloneConfigManager initialized without environment dependencies")
    
    def _initialize_default_configs(self):
        """初始化默认配置"""
        # 设置运行时配置
        self._runtime_config = {
            "timing": {
                "heartbeat_interval_seconds": self.config.heartbeat_interval_seconds,
                "http_timeout_seconds": self.config.http_timeout_seconds,
                "reconnection_interval_seconds": self.config.reconnection_interval_seconds,
                "cleanup_interval_seconds": self.config.cleanup_interval_seconds
            },
            "network": {
                "streamable_http_endpoint": self.config.streamable_http_endpoint,
                "default_transport": self.config.default_transport
            },
            "environment": {
                "isolated": self.config.isolated_environment,
                "base_env": self.config.base_environment.copy()
            }
        }
        
        # 设置已知服务配置
        self._service_configs = deepcopy(self.config.known_services)
    
    def get_timing_config(self) -> Dict[str, int]:
        """获取时间配置"""
        return self._runtime_config["timing"]
    
    def get_network_config(self) -> Dict[str, str]:
        """获取网络配置"""
        return self._runtime_config["network"]
    
    def get_environment_config(self) -> Dict[str, Any]:
        """获取环境配置"""
        return self._runtime_config["environment"]
    
    def get_service_config(self, service_name: str) -> Optional[Dict[str, Any]]:
        """获取服务配置"""
        return self._service_configs.get(service_name)
    
    def add_service_config(self, service_name: str, config: Dict[str, Any]):
        """添加服务配置"""
        self._service_configs[service_name] = deepcopy(config)
        logger.info(f"Added service config for: {service_name}")
    
    def get_all_service_configs(self) -> Dict[str, Dict[str, Any]]:
        """获取所有服务配置"""
        return deepcopy(self._service_configs)
    
    def get_mcp_config(self) -> Dict[str, Any]:
        """获取MCP格式的配置"""
        return {
            "mcpServers": deepcopy(self._service_configs),
            "version": "1.0.0",
            "description": "MCPStore standalone configuration"
        }
    
    def update_config(self, **kwargs):
        """更新配置"""
        for key, value in kwargs.items():
            if hasattr(self.config, key):
                setattr(self.config, key, value)
                logger.info(f"Updated config: {key} = {value}")
        
        # 重新初始化配置
        self._initialize_default_configs()
    
    def get_isolated_environment(self, custom_env: Optional[Dict[str, str]] = None) -> Dict[str, str]:
        """
        获取隔离的环境变量
        
        Args:
            custom_env: 自定义环境变量
            
        Returns:
            隔离的环境变量字典
        """
        if not self.config.isolated_environment:
            # 如果不使用隔离环境，返回空字典（让FastMCP处理）
            return custom_env or {}
        
        # 使用基础环境
        env = self.config.base_environment.copy()
        
        # 添加自定义环境变量
        if custom_env:
            env.update(custom_env)
        
        return env
    
    def get_config_paths(self) -> Dict[str, Optional[str]]:
        """获取配置文件路径"""
        return {
            "config_dir": self.config.config_dir,
            "mcp_config_file": self.config.mcp_config_file,
            "client_services_file": self.config.client_services_file,
            "agent_clients_file": self.config.agent_clients_file
        }
    
    def is_file_based(self) -> bool:
        """检查是否使用文件配置"""
        return self.config.config_dir is not None or self.config.mcp_config_file is not None

class StandaloneConfigBuilder:
    """独立配置构建器 - 提供流畅的配置构建接口"""
    
    def __init__(self):
        self._config = StandaloneConfig()
    
    def with_timing(self, heartbeat: int = None, timeout: int = None, reconnection: int = None) -> 'StandaloneConfigBuilder':
        """设置时间配置"""
        if heartbeat is not None:
            self._config.heartbeat_interval_seconds = heartbeat
        if timeout is not None:
            self._config.http_timeout_seconds = timeout
        if reconnection is not None:
            self._config.reconnection_interval_seconds = reconnection
        return self
    
    def with_network(self, endpoint: str = None, transport: str = None) -> 'StandaloneConfigBuilder':
        """设置网络配置"""
        if endpoint is not None:
            self._config.streamable_http_endpoint = endpoint
        if transport is not None:
            self._config.default_transport = transport
        return self
    
    def with_files(self, config_dir: str = None, mcp_file: str = None) -> 'StandaloneConfigBuilder':
        """设置文件配置"""
        if config_dir is not None:
            self._config.config_dir = config_dir
        if mcp_file is not None:
            self._config.mcp_config_file = mcp_file
        return self
    
    def with_service(self, name: str, config: Dict[str, Any]) -> 'StandaloneConfigBuilder':
        """添加服务配置"""
        self._config.known_services[name] = config
        return self
    
    def with_environment(self, isolated: bool = None, base_env: Dict[str, str] = None) -> 'StandaloneConfigBuilder':
        """设置环境配置"""
        if isolated is not None:
            self._config.isolated_environment = isolated
        if base_env is not None:
            self._config.base_environment = base_env
        return self
    
    def with_logging(self, level: str = None, debug: bool = None) -> 'StandaloneConfigBuilder':
        """设置日志配置"""
        if level is not None:
            self._config.log_level = level
        if debug is not None:
            self._config.enable_debug = debug
        return self
    
    def build(self) -> StandaloneConfig:
        """构建配置"""
        return deepcopy(self._config)

# === 预定义配置模板 ===

def create_minimal_config() -> StandaloneConfig:
    """创建最小配置 - 只包含基本功能"""
    return StandaloneConfigBuilder().build()

def create_development_config() -> StandaloneConfig:
    """创建开发配置 - 包含调试功能"""
    return (StandaloneConfigBuilder()
            .with_timing(heartbeat=30, timeout=10, reconnection=60)
            .with_logging(level="DEBUG", debug=True)
            .build())

def create_production_config() -> StandaloneConfig:
    """创建生产配置 - 优化性能和稳定性"""
    return (StandaloneConfigBuilder()
            .with_timing(heartbeat=120, timeout=60, reconnection=600)
            .with_environment(isolated=True)
            .with_logging(level="WARNING", debug=False)
            .build())

def create_studio_config() -> StandaloneConfig:
    """创建Studio配置 - 针对长连接优化"""
    return (StandaloneConfigBuilder()
            .with_timing(heartbeat=300, timeout=120, reconnection=1800)
            .with_network(transport="streamable-http")
            .with_environment(isolated=False)  # Studio可能需要访问系统环境
            .build())

# === 全局配置实例 ===
_global_config_manager: Optional[StandaloneConfigManager] = None

def get_global_config() -> StandaloneConfigManager:
    """获取全局配置管理器"""
    global _global_config_manager
    if _global_config_manager is None:
        _global_config_manager = StandaloneConfigManager()
    return _global_config_manager

def set_global_config(config: Union[StandaloneConfig, StandaloneConfigManager]):
    """设置全局配置"""
    global _global_config_manager
    if isinstance(config, StandaloneConfig):
        _global_config_manager = StandaloneConfigManager(config)
    else:
        _global_config_manager = config
    logger.info("Global standalone config updated")

def reset_global_config():
    """重置全局配置"""
    global _global_config_manager
    _global_config_manager = None
    logger.info("Global standalone config reset")



"""
MCPStore 通用响应模型

提供统一的响应格式，减少重复的响应模型定义。
"""

from pydantic import BaseModel, Field
from typing import Optional, Any, List, Dict, Generic, TypeVar

# 泛型类型变量
T = TypeVar('T')

class BaseResponse(BaseModel):
    """统一的基础响应模型"""
    success: bool = Field(..., description="操作是否成功")
    message: Optional[str] = Field(None, description="响应消息")

class APIResponse(BaseResponse):
    """通用API响应模型"""
    data: Optional[Any] = Field(None, description="响应数据")
    metadata: Optional[Dict[str, Any]] = Field(None, description="元数据信息")
    execution_info: Optional[Dict[str, Any]] = Field(None, description="执行信息")

class ListResponse(BaseResponse, Generic[T]):
    """列表响应模型"""
    items: List[T] = Field(..., description="数据项列表")
    total: int = Field(..., description="总数量")

class DataResponse(BaseResponse, Generic[T]):
    """单个数据项响应模型"""
    data: T = Field(..., description="数据项")

class RegistrationResponse(BaseResponse):
    """注册操作响应模型"""
    client_id: str = Field(..., description="客户端ID")
    service_names: List[str] = Field(..., description="服务名列表")
    config: Dict[str, Any] = Field(..., description="配置信息")

class ExecutionResponse(BaseResponse):
    """执行操作响应模型"""
    result: Optional[Any] = Field(None, description="执行结果")
    error: Optional[str] = Field(None, description="错误信息")

class ConfigResponse(BaseResponse):
    """配置响应模型"""
    client_id: str = Field(..., description="客户端ID")
    config: Dict[str, Any] = Field(..., description="配置信息")

class HealthResponse(BaseResponse):
    """健康检查响应模型"""
    service_name: str = Field(..., description="服务名称")
    status: str = Field(..., description="健康状态")
    last_check: Optional[str] = Field(None, description="最后检查时间")

# 这些别名已被删除，直接使用新的统一响应模型



"""
MCPStore - 智能体工具服务商店
提供简单易用的MCP工具管理和调用功能
"""

from mcpstore.core.store import MCPStore
from mcpstore.config.config import LoggingConfig

__version__ = "0.5.0"
__all__ = ["MCPStore", "LoggingConfig"]
